@@ -404,7 +404,7 @@ update_transaction_status (const struct Inquiry *w,
   if (qs < 0)
   {
     GNUNET_break (0);
-    global_ret = 1;
+    global_ret = EXIT_FAILURE;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -519,6 +519,11 @@ shutdown_task (void *cls)
       TALER_EXCHANGE_get_keys_cancel (e->conn);
       e->conn = NULL;
     }
+    if (NULL != e->keys)
+    {
+      TALER_EXCHANGE_keys_decref (e->keys);
+      e->keys = NULL;
+    }
     if (NULL != e->retry_task)
     {
       GNUNET_SCHEDULER_cancel (e->retry_task);
@@ -617,7 +622,7 @@ check_wire_fee (struct Inquiry *w,
         TALER_amount_cmp_currency (&fees.wire,
                                    wire_fee)) ||
        (0 > TALER_amount_cmp (&fees.wire,
-                               wire_fee)) )
+                              wire_fee)) )
   {
     GNUNET_break_op (0);
     GNUNET_free (wire_method);
@@ -811,11 +816,11 @@ wire_transfer_cb (void *cls,
   {
     /* Always report on DB error as well to enable diagnostics */
     GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
-    global_ret = 1;
+    global_ret = EXIT_FAILURE;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  if (0 == qs)
+  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Transfer already known. Ignoring duplicate.\n");
@@ -1165,6 +1170,7 @@ run (void *cls,
   {
     GNUNET_break (0);
     GNUNET_SCHEDULER_shutdown ();
+    global_ret = EXIT_NO_RESTART;
     return;
   }
   if (NULL ==
@@ -1173,6 +1179,7 @@ run (void *cls,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to initialize DB subsystem\n");
     GNUNET_SCHEDULER_shutdown ();
+    global_ret = EXIT_NOTCONFIGURED;
     return;
   }
   if (GNUNET_OK !=
@@ -1181,6 +1188,7 @@ run (void *cls,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to connect to database\n");
     GNUNET_SCHEDULER_shutdown ();
+    global_ret = EXIT_NO_RESTART;
     return;
   }
   {
@@ -1212,6 +1220,8 @@ main (int argc,
       char *const *argv)
 {
   struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_timetravel ('T',
+                                     "timetravel"),
     GNUNET_GETOPT_option_flag ('t',
                                "test",
                                "run in test mode and exit when idle",
