@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2022 Taler Systems SA
+  (C) 2014-2024 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -31,7 +31,6 @@
 #include "taler-merchant-httpd_config.h"
 #include "taler-merchant-httpd_exchanges.h"
 #include "taler-merchant-httpd_get-orders-ID.h"
-#include "taler-merchant-httpd_get-rewards-ID.h"
 #include "taler-merchant-httpd_mhd.h"
 #include "taler-merchant-httpd_private-delete-account-ID.h"
 #include "taler-merchant-httpd_private-delete-instances-ID.h"
@@ -39,8 +38,8 @@
 #include "taler-merchant-httpd_private-delete-products-ID.h"
 #include "taler-merchant-httpd_private-delete-orders-ID.h"
 #include "taler-merchant-httpd_private-delete-otp-devices-ID.h"
-#include "taler-merchant-httpd_private-delete-reserves-ID.h"
 #include "taler-merchant-httpd_private-delete-templates-ID.h"
+#include "taler-merchant-httpd_private-delete-token-families-SLUG.h"
 #include "taler-merchant-httpd_private-delete-transfers-ID.h"
 #include "taler-merchant-httpd_private-delete-webhooks-ID.h"
 #include "taler-merchant-httpd_private-get-accounts.h"
@@ -54,12 +53,10 @@
 #include "taler-merchant-httpd_private-get-orders-ID.h"
 #include "taler-merchant-httpd_private-get-otp-devices.h"
 #include "taler-merchant-httpd_private-get-otp-devices-ID.h"
-#include "taler-merchant-httpd_private-get-reserves.h"
-#include "taler-merchant-httpd_private-get-reserves-ID.h"
-#include "taler-merchant-httpd_private-get-rewards-ID.h"
-#include "taler-merchant-httpd_private-get-rewards.h"
 #include "taler-merchant-httpd_private-get-templates.h"
 #include "taler-merchant-httpd_private-get-templates-ID.h"
+#include "taler-merchant-httpd_private-get-token-families.h"
+#include "taler-merchant-httpd_private-get-token-families-SLUG.h"
 #include "taler-merchant-httpd_private-get-transfers.h"
 #include "taler-merchant-httpd_private-get-webhooks.h"
 #include "taler-merchant-httpd_private-get-webhooks-ID.h"
@@ -69,6 +66,7 @@
 #include "taler-merchant-httpd_private-patch-otp-devices-ID.h"
 #include "taler-merchant-httpd_private-patch-products-ID.h"
 #include "taler-merchant-httpd_private-patch-templates-ID.h"
+#include "taler-merchant-httpd_private-patch-token-families-SLUG.h"
 #include "taler-merchant-httpd_private-patch-webhooks-ID.h"
 #include "taler-merchant-httpd_private-post-account.h"
 #include "taler-merchant-httpd_private-post-instances.h"
@@ -79,9 +77,8 @@
 #include "taler-merchant-httpd_private-post-orders-ID-refund.h"
 #include "taler-merchant-httpd_private-post-products.h"
 #include "taler-merchant-httpd_private-post-products-ID-lock.h"
-#include "taler-merchant-httpd_private-post-reserves.h"
-#include "taler-merchant-httpd_private-post-reserves-ID-authorize-reward.h"
 #include "taler-merchant-httpd_private-post-templates.h"
+#include "taler-merchant-httpd_private-post-token-families.h"
 #include "taler-merchant-httpd_private-post-transfers.h"
 #include "taler-merchant-httpd_private-post-webhooks.h"
 #include "taler-merchant-httpd_post-orders-ID-abort.h"
@@ -90,8 +87,6 @@
 #include "taler-merchant-httpd_post-orders-ID-pay.h"
 #include "taler-merchant-httpd_post-using-templates.h"
 #include "taler-merchant-httpd_post-orders-ID-refund.h"
-#include "taler-merchant-httpd_post-rewards-ID-pickup.h"
-#include "taler-merchant-httpd_reserves.h"
 #include "taler-merchant-httpd_spa.h"
 #include "taler-merchant-httpd_statics.h"
 
@@ -187,7 +182,7 @@ static struct GNUNET_CURL_RescheduleContext *merchant_curl_rc;
 /**
  * Global return code
  */
-static int result;
+static int global_ret;
 
 /**
  * Our configuration.
@@ -400,13 +395,10 @@ do_shutdown (void *cls)
 {
   (void) cls;
   TMH_force_orders_resume ();
-  TMH_force_reward_resume ();
   TMH_force_ac_resume ();
   TMH_force_pc_resume ();
   TMH_force_kyc_resume ();
-  TMH_force_rc_resume ();
   TMH_force_gorc_resume ();
-  TMH_force_reward_pickup_resume ();
   TMH_force_wallet_get_order_resume ();
   TMH_force_wallet_refund_order_resume ();
   {
@@ -416,7 +408,6 @@ do_shutdown (void *cls)
     if (NULL != mhd)
       MHD_stop_daemon (mhd);
   }
-  TMH_RESERVES_done ();
   if (NULL != instance_eh)
   {
     TMH_db->event_listen_cancel (instance_eh);
@@ -506,6 +497,7 @@ handle_mhd_completion_callback (void *cls,
     json_decref (hc->request_body);
   if (NULL != hc->instance)
     TMH_instance_decref (hc->instance);
+  GNUNET_free (hc->full_url);
   GNUNET_free (hc);
   *con_cls = NULL;
 }
@@ -617,7 +609,9 @@ spa_redirect (const struct TMH_RequestHandler *rh,
                 MHD_add_response_header (response,
                                          MHD_HTTP_HEADER_CONTENT_TYPE,
                                          "text/plain"));
-  if (NULL == hc->instance)
+  if ( (NULL == hc->instance) ||
+       (0 == strcmp ("default",
+                     hc->instance->settings.id)) )
     dst = GNUNET_strdup ("/webui/");
   else
     GNUNET_asprintf (&dst,
@@ -742,6 +736,29 @@ prefix_match (const struct TMH_RequestHandler *rh,
 }
 
 
+/**
+ * Function called first by MHD with the full URL.
+ *
+ * @param cls NULL
+ * @param full_url the full URL
+ * @param con MHD connection object
+ * @return our handler context
+ */
+static void *
+full_url_track_callback (void *cls,
+                         const char *full_url,
+                         struct MHD_Connection *con)
+{
+  struct TMH_HandlerContext *hc;
+
+  hc = GNUNET_new (struct TMH_HandlerContext);
+  GNUNET_async_scope_fresh (&hc->async_scope_id);
+  GNUNET_SCHEDULER_begin_async_scope (&hc->async_scope_id);
+  hc->full_url = GNUNET_strdup (full_url);
+  return hc;
+}
+
+
 /**
  * A client has requested the given url using the given method
  * (#MHD_HTTP_METHOD_GET, #MHD_HTTP_METHOD_PUT,
@@ -777,7 +794,8 @@ prefix_match (const struct TMH_RequestHandler *rh,
  *        If necessary, this state can be cleaned up in the
  *        global #MHD_RequestCompletedCallback (which
  *        can be set with the #MHD_OPTION_NOTIFY_COMPLETED).
- *        Initially, `*con_cls` will be NULL.
+ *        Initially, `*con_cls` will be set up by the
+ *        full_url_track_callback().
  * @return #MHD_YES if the connection was handled successfully,
  *         #MHD_NO if the socket must be closed due to a serious
  *         error while handling the request
@@ -858,7 +876,7 @@ url_handler (void *cls,
       /* Body should be pretty small. */
       .max_upload = 1024 * 1024
     },
-    /* POST /kyc: */
+    /* GET /kyc: */
     {
       .url_prefix = "/instances/",
       .url_suffix = "kyc",
@@ -1027,73 +1045,6 @@ url_handler (void *cls,
       .allow_deleted_instance = true,
       .handler = &TMH_private_delete_orders_ID
     },
-    /* POST /reserves: */
-    {
-      .url_prefix = "/reserves",
-      .method = MHD_HTTP_METHOD_POST,
-      .handler = &TMH_private_post_reserves,
-      /* the body should be pretty small, allow 1 MB of upload
-         to set a conservative bound for sane wallets */
-      .max_upload = 1024 * 1024
-    },
-    /* DELETE /reserves/$ID: */
-    {
-      .url_prefix = "/reserves/",
-      .have_id_segment = true,
-      .allow_deleted_instance = true,
-      .method = MHD_HTTP_METHOD_DELETE,
-      .handler = &TMH_private_delete_reserves_ID
-    },
-    /* POST /reserves/$ID/authorize-reward: */
-    {
-      .url_prefix = "/reserves/",
-      .url_suffix = "authorize-reward",
-      .have_id_segment = true,
-      .method = MHD_HTTP_METHOD_POST,
-      .handler = &TMH_private_post_reserves_ID_authorize_reward,
-      /* the body should be pretty small, allow 1 MB of upload
-         to set a conservative bound for sane wallets */
-      .max_upload = 1024 * 1024
-    },
-    /* POST /rewards: */
-    {
-      .url_prefix = "/rewards",
-      .method = MHD_HTTP_METHOD_POST,
-      .handler = &TMH_private_post_rewards,
-      /* the body should be pretty small, allow 1 MB of upload
-         to set a conservative bound for sane wallets */
-      .max_upload = 1024 * 1024
-    },
-    /* GET /rewards: */
-    {
-      .url_prefix = "/rewards",
-      .allow_deleted_instance = true,
-      .method = MHD_HTTP_METHOD_GET,
-      .handler = &TMH_private_get_rewards
-    },
-    /* GET /rewards/$ID: */
-    {
-      .url_prefix = "/rewards/",
-      .method = MHD_HTTP_METHOD_GET,
-      .allow_deleted_instance = true,
-      .have_id_segment = true,
-      .handler = &TMH_private_get_rewards_ID
-    },
-    /* GET /reserves: */
-    {
-      .url_prefix = "/reserves",
-      .allow_deleted_instance = true,
-      .method = MHD_HTTP_METHOD_GET,
-      .handler = &TMH_private_get_reserves
-    },
-    /* GET /reserves/$ID: */
-    {
-      .url_prefix = "/reserves/",
-      .allow_deleted_instance = true,
-      .have_id_segment = true,
-      .method = MHD_HTTP_METHOD_GET,
-      .handler = &TMH_private_get_reserves_ID
-    },
     /* POST /transfers: */
     {
       .url_prefix = "/transfers",
@@ -1257,7 +1208,7 @@ url_handler (void *cls,
     },
     /* PATCH /accounts/$H_WIRE: */
     {
-      .url_prefix = "/accounts",
+      .url_prefix = "/accounts/",
       .method = MHD_HTTP_METHOD_PATCH,
       .handler = &TMH_private_patch_accounts_ID,
       .have_id_segment = true,
@@ -1300,16 +1251,64 @@ url_handler (void *cls,
       .method = MHD_HTTP_METHOD_DELETE,
       .handler = &TMH_private_delete_instances_ID_token,
     },
+    /* GET /tokenfamilies: */
+    {
+      .url_prefix = "/tokenfamilies",
+      .method = MHD_HTTP_METHOD_GET,
+      .handler = &TMH_private_get_tokenfamilies
+    },
+    /* POST /tokenfamilies: */
+    {
+      .url_prefix = "/tokenfamilies",
+      .method = MHD_HTTP_METHOD_POST,
+      .handler = &TMH_private_post_token_families
+    },
+    /* GET /tokenfamilies/$SLUG/: */
+    {
+      .url_prefix = "/tokenfamilies/",
+      .method = MHD_HTTP_METHOD_GET,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_get_tokenfamilies_SLUG
+    },
+    /* DELETE /tokenfamilies/$SLUG/: */
+    {
+      .url_prefix = "/tokenfamilies/",
+      .method = MHD_HTTP_METHOD_DELETE,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_delete_token_families_SLUG
+    },
+    /* PATCH /tokenfamilies/$SLUG/: */
+    {
+      .url_prefix = "/tokenfamilies/",
+      .method = MHD_HTTP_METHOD_PATCH,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_patch_token_family_SLUG,
+    },
     {
       .url_prefix = NULL
     }
   };
   static struct TMH_RequestHandler public_handlers[] = {
     {
+      /* for "default" instance, it does not even
+         have to exist before we give the WebUI */
       .url_prefix = "/",
       .method = MHD_HTTP_METHOD_GET,
       .mime_type = "text/html",
       .skip_instance = true,
+      .default_only = true,
+      .handler = &spa_redirect,
+      .response_code = MHD_HTTP_FOUND
+    },
+    {
+      /* for "normal" instance,s they must exist
+         before we give the WebUI */
+      .url_prefix = "/",
+      .method = MHD_HTTP_METHOD_GET,
+      .mime_type = "text/html",
       .handler = &spa_redirect,
       .response_code = MHD_HTTP_FOUND
     },
@@ -1408,26 +1407,6 @@ url_handler (void *cls,
       .have_id_segment = true,
       .handler = &TMH_get_orders_ID
     },
-    /* GET /rewards/$ID: */
-    {
-      .url_prefix = "/rewards/",
-      .method = MHD_HTTP_METHOD_GET,
-      .allow_deleted_instance = true,
-      .have_id_segment = true,
-      .handler = &TMH_get_rewards_ID
-    },
-    /* POST /rewards/$ID/pickup: */
-    {
-      .url_prefix = "/rewards/",
-      .method = MHD_HTTP_METHOD_POST,
-      .have_id_segment = true,
-      .allow_deleted_instance = true,
-      .url_suffix = "pickup",
-      .handler = &TMH_post_rewards_ID_pickup,
-      /* wallet may give us many coins to sign, allow 1 MB of upload
-         to set a conservative bound for sane wallets */
-      .max_upload = 1024 * 1024
-    },
     /* GET /static/ *: */
     {
       .url_prefix = "/static/",
@@ -1459,7 +1438,7 @@ url_handler (void *cls,
 
   (void) cls;
   (void) version;
-  if (NULL != hc)
+  if (NULL != hc->url)
   {
     /* MHD calls us again for a request, for first call
        see 'else' case below */
@@ -1507,10 +1486,6 @@ url_handler (void *cls,
                             connection,
                             hc);
   }
-  hc = GNUNET_new (struct TMH_HandlerContext);
-  *con_cls = hc;
-  GNUNET_async_scope_fresh (&hc->async_scope_id);
-  GNUNET_SCHEDULER_begin_async_scope (&hc->async_scope_id);
   hc->url = url;
   {
     const char *correlation_id;
@@ -1564,6 +1539,37 @@ url_handler (void *cls,
       else
         instance_id = GNUNET_strndup (istart,
                                       slash - istart);
+      if (0 == strcmp (instance_id,
+                       "default"))
+      {
+        MHD_RESULT ret;
+        struct MHD_Response *response;
+        const char *rstart = hc->full_url + strlen (instance_prefix);
+        const char *rslash = strchr (rstart, '/');
+
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "Client used deprecated '/instances/default/' path. Redirecting to modern path\n");
+
+        response
+          = MHD_create_response_from_buffer (0,
+                                             NULL,
+                                             MHD_RESPMEM_PERSISTENT);
+        TALER_MHD_add_global_headers (response);
+        if (MHD_NO ==
+            MHD_add_response_header (response,
+                                     MHD_HTTP_HEADER_LOCATION,
+                                     rslash))
+        {
+          GNUNET_break (0);
+          MHD_destroy_response (response);
+          return MHD_NO;
+        }
+        ret = MHD_queue_response (connection,
+                                  MHD_HTTP_PERMANENT_REDIRECT,
+                                  response);
+        MHD_destroy_response (response);
+        return ret;
+      }
       hc->instance = TMH_lookup_instance (instance_id);
       if ( (NULL == hc->instance) &&
            (0 == strcmp ("default",
@@ -1632,7 +1638,11 @@ url_handler (void *cls,
                             "/private")) )
     {
       handlers = private_handlers;
-      url += strlen (private_prefix) - 1;
+      if (0 == strcmp (url,
+                       "/private"))
+        url = "/";
+      else
+        url += strlen (private_prefix) - 1;
     }
     else
     {
@@ -2087,7 +2097,7 @@ load_instances (void *cls,
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed initialization. Check database setup.\n");
-    result = EXIT_FAILURE;
+    global_ret = EXIT_NOPERMISSION;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2152,10 +2162,10 @@ run (void *cls,
                       RFC_8959_PREFIX,
                       strlen (RFC_8959_PREFIX))) )
   {
-    fprintf (stderr,
-             "Authentication token does not start with `%s' prefix\n",
-             RFC_8959_PREFIX);
-    result = GNUNET_SYSERR;
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Authentication token does not start with `%s' prefix\n",
+                RFC_8959_PREFIX);
+    global_ret = EXIT_NOTCONFIGURED;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2167,7 +2177,7 @@ run (void *cls,
     go |= TALER_MHD_GO_FORCE_CONNECTION_CLOSE;
   TALER_MHD_setup (go);
 
-  result = GNUNET_SYSERR;
+  global_ret = EXIT_SUCCESS;
   GNUNET_SCHEDULER_add_shutdown (&do_shutdown,
                                  NULL);
 
@@ -2177,7 +2187,7 @@ run (void *cls,
   if (NULL == TMH_curl_ctx)
   {
     GNUNET_break (0);
-    result = GNUNET_SYSERR;
+    global_ret = EXIT_NO_RESTART;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2189,10 +2199,11 @@ run (void *cls,
   GNUNET_CURL_enable_async_scope_header (TMH_curl_ctx,
                                          "Taler-Correlation-Id");
 
-  if (GNUNET_OK !=
+  if (GNUNET_SYSERR ==
       TALER_config_get_currency (cfg,
                                  &TMH_currency))
   {
+
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2225,7 +2236,7 @@ run (void *cls,
                                              &TMH_base_url))
   {
     if (! TALER_is_web_url (TMH_base_url))
-   {
+    {
       GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
                                  "merchant",
                                  "BASE_URL",
@@ -2300,8 +2311,6 @@ run (void *cls,
   load_instances (NULL,
                   NULL,
                   0);
-  /* start watching reserves */
-  TMH_RESERVES_init ();
   fh = TALER_MHD_bind (cfg,
                        "merchant",
                        &port);
@@ -2321,6 +2330,8 @@ run (void *cls,
                             NULL, NULL,
                             &url_handler, NULL,
                             MHD_OPTION_LISTEN_SOCKET, fh,
+                            MHD_OPTION_URI_LOG_CALLBACK,
+                            &full_url_track_callback, NULL,
                             MHD_OPTION_NOTIFY_COMPLETED,
                             &handle_mhd_completion_callback, NULL,
                             MHD_OPTION_CONNECTION_TIMEOUT,
@@ -2333,7 +2344,7 @@ run (void *cls,
       GNUNET_SCHEDULER_shutdown ();
       return;
     }
-    result = GNUNET_OK;
+    global_ret = EXIT_SUCCESS;
     TALER_MHD_daemon_start (mhd);
   }
 }
@@ -2377,5 +2388,5 @@ main (int argc,
     return EXIT_INVALIDARGUMENT;
   if (GNUNET_NO == res)
     return EXIT_SUCCESS;
-  return (GNUNET_OK == result) ? EXIT_SUCCESS : 1;
+  return global_ret;
 }
