@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2019, 2020, 2021 Taler Systems SA
+  (C) 2019, 2020, 2021, 2023, 2024 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -42,7 +42,38 @@
  * #MERCHANT_PROTOCOL_CURRENT and #MERCHANT_PROTOCOL_AGE in
  * merchant_api_config.c!
  */
-#define MERCHANT_PROTOCOL_VERSION "5:0:1"
+#define MERCHANT_PROTOCOL_VERSION "9:0:5"
+
+
+/**
+ * Callback on an exchange known to us. Does not warrant
+ * that the "keys" information is actually available for
+ * @a exchange.
+ *
+ * @param cls closure with `json_t *` array to expand
+ * @param url base URL of the exchange
+ * @param exchange internal handle for the exchange
+ */
+static void
+add_exchange (void *cls,
+              const char *url,
+              const struct TMH_Exchange *exchange)
+{
+  json_t *xa = cls;
+  json_t *xi;
+
+  xi = GNUNET_JSON_PACK (
+    GNUNET_JSON_pack_data_auto ("master_pub",
+                                TMH_EXCHANGES_get_master_pub (exchange)),
+    GNUNET_JSON_pack_string ("currency",
+                             TMH_EXCHANGES_get_currency (exchange)),
+    GNUNET_JSON_pack_string ("base_url",
+                             url));
+  GNUNET_assert (NULL != xi);
+  GNUNET_assert (0 ==
+                 json_array_append_new (xa,
+                                        xi));
+}
 
 
 MHD_RESULT
@@ -57,9 +88,13 @@ MH_handler_config (struct TMH_RequestHandler *rh,
   if (NULL == response)
   {
     json_t *specs = json_object ();
+    json_t *exchanges = json_array ();
 
     GNUNET_assert (NULL != specs);
-    for (unsigned int i=0;i<TMH_num_cspecs;i++)
+    GNUNET_assert (NULL != exchanges);
+    TMH_exchange_get_trusted (&add_exchange,
+                              exchanges);
+    for (unsigned int i = 0; i<TMH_num_cspecs; i++)
     {
       const struct TALER_CurrencySpecification *cspec = &TMH_cspecs[i];
 
@@ -67,13 +102,18 @@ MH_handler_config (struct TMH_RequestHandler *rh,
         GNUNET_assert (0 ==
                        json_object_set_new (specs,
                                             cspec->currency,
-                                            TALER_CONFIG_currency_specs_to_json (cspec)));
+                                            TALER_CONFIG_currency_specs_to_json (
+                                              cspec)));
     }
     response = TALER_MHD_MAKE_JSON_PACK (
       GNUNET_JSON_pack_string ("currency",
                                TMH_currency),
       GNUNET_JSON_pack_object_steal ("currencies",
                                      specs),
+      GNUNET_JSON_pack_array_steal ("exchanges",
+                                    exchanges),
+      GNUNET_JSON_pack_string ("implementation",
+                               "urn:net:taler:specs:merchant:c-reference"),
       GNUNET_JSON_pack_string ("name",
                                "taler-merchant"),
       GNUNET_JSON_pack_string ("version",
