@@ -161,6 +161,12 @@ struct GetOrderData
    */
   enum GNUNET_GenericReturnValue suspended;
 
+  /**
+   * Set to YES if refunded orders should be included when
+   * doing repurchase detection.
+   */
+  enum TALER_EXCHANGE_YesNoAll allow_refunded_for_repurchase;
+  
   /**
    * Set to true if the client passed 'h_contract'.
    */
@@ -795,6 +801,7 @@ god_cleanup (void *cls)
 }
 
 
+// FIXME: this function should probably be refactored...
 MHD_RESULT
 TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                    struct MHD_Connection *connection,
@@ -827,6 +834,14 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                       "token",
                                       &god->claim_token,
                                       god->claim_token_provided);
+    if (! (TALER_arg_to_yna (connection,
+                             "allow_refunded_for_repurchase",
+                             TALER_EXCHANGE_YNA_NO,
+                             &god->allow_refunded_for_repurchase)) )
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                         "allow_refunded_for_repurchase");
     god->session_id = MHD_lookup_connection_value (connection,
                                                    MHD_GET_ARGUMENT_KIND,
                                                    "session_id");
@@ -950,7 +965,6 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   if (NULL == god->contract_terms)
   {
     uint64_t order_serial;
-    bool paid = false;
     struct TALER_ClaimTokenP db_claim_token;
 
     qs = TMH_db->lookup_contract_terms (TMH_db->cls,
@@ -958,7 +972,6 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                         order_id,
                                         &god->contract_terms,
                                         &order_serial,
-                                        &paid,
                                         &db_claim_token);
     if (0 > qs)
     {
@@ -1229,6 +1242,8 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                               hc->instance->settings.id,
                                               god->fulfillment_url,
                                               god->session_id,
+                                              TALER_EXCHANGE_YNA_NO !=
+                                              god->allow_refunded_for_repurchase,
                                               &already_paid_order_id);
     if (qs < 0)
     {
