@@ -980,3 +980,113 @@ TMH_exchange_accounts_by_method (
   }
   return emc.accounts;
 }
+
+
+char *
+TMH_make_order_status_url (struct MHD_Connection *con,
+                           const char *order_id,
+                           const char *session_id,
+                           const char *instance_id,
+                           struct TALER_ClaimTokenP *claim_token,
+                           struct TALER_PrivateContractHashP *h_contract)
+{
+  struct GNUNET_Buffer buf;
+  /* Number of query parameters written so far */
+  unsigned int num_qp = 0;
+
+  GNUNET_assert (NULL != instance_id);
+  GNUNET_assert (NULL != order_id);
+  if (GNUNET_OK !=
+      TMH_base_url_by_connection (con,
+                                  instance_id,
+                                  &buf))
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  GNUNET_buffer_write_path (&buf,
+                            "/orders");
+  GNUNET_buffer_write_path (&buf,
+                            order_id);
+  if ( (NULL != claim_token) &&
+       (! GNUNET_is_zero (claim_token)) )
+  {
+    /* 'token=' for human readability */
+    GNUNET_buffer_write_str (&buf,
+                             "?token=");
+    GNUNET_buffer_write_data_encoded (&buf,
+                                      (char *) claim_token,
+                                      sizeof (*claim_token));
+    num_qp++;
+  }
+
+  if (NULL != session_id)
+  {
+    if (num_qp > 0)
+      GNUNET_buffer_write_str (&buf,
+                               "&session_id=");
+    else
+      GNUNET_buffer_write_str (&buf,
+                               "?session_id=");
+    GNUNET_buffer_write_str (&buf,
+                             session_id);
+    num_qp++;
+  }
+
+  if (NULL != h_contract)
+  {
+    if (num_qp > 0)
+      GNUNET_buffer_write_str (&buf,
+                               "&h_contract=");
+    else
+      GNUNET_buffer_write_str (&buf,
+                               "?h_contract=");
+    GNUNET_buffer_write_data_encoded (&buf,
+                                      (char *) h_contract,
+                                      sizeof (*h_contract));
+  }
+
+  return GNUNET_buffer_reap_str (&buf);
+}
+
+
+char *
+TMH_make_taler_pay_uri (struct MHD_Connection *con,
+                        const char *order_id,
+                        const char *session_id,
+                        const char *instance_id,
+                        struct TALER_ClaimTokenP *claim_token)
+{
+  struct GNUNET_Buffer buf;
+
+  GNUNET_assert (NULL != instance_id);
+  GNUNET_assert (NULL != order_id);
+  if (GNUNET_OK !=
+      TMH_taler_uri_by_connection (con,
+                                   "pay",
+                                   instance_id,
+                                   &buf))
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  GNUNET_buffer_write_path (&buf,
+                            order_id);
+  GNUNET_buffer_write_path (&buf,
+                            (NULL == session_id)
+                            ? ""
+                            : session_id);
+  if ( (NULL != claim_token) &&
+       (! GNUNET_is_zero (claim_token)))
+  {
+    /* Just 'c=' because this goes into QR
+       codes, so this is more compact. */
+    GNUNET_buffer_write_str (&buf,
+                             "?c=");
+    GNUNET_buffer_write_data_encoded (&buf,
+                                      (char *) claim_token,
+                                      sizeof (struct TALER_ClaimTokenP));
+  }
+
+  return GNUNET_buffer_reap_str (&buf);
+}
