@@ -885,7 +885,7 @@ parse_abort (struct MHD_Connection *connection,
         TALER_JSON_spec_amount ("contribution",
                                 TMH_currency,
                                 &rd->amount_with_fee),
-        GNUNET_JSON_spec_string ("exchange_url",
+        TALER_JSON_spec_web_url ("exchange_url",
                                  &exchange_url),
         GNUNET_JSON_spec_fixed_auto ("coin_pub",
                                      &rd->coin_pub),
