@@ -581,14 +581,12 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
     {
       json_t *contract_terms;
       uint64_t order_serial;
-      bool paid;
 
       qs = TMH_db->lookup_contract_terms (TMH_db->cls,
                                           hc->instance->settings.id,
                                           hc->infix,
                                           &contract_terms,
                                           &order_serial,
-                                          &paid,
                                           NULL);
       if (0 > qs)
       {
