@@ -39,13 +39,13 @@
  * How long should clients cache a KYC failure response?
  */
 #define EXPIRATION_KYC_FAILURE GNUNET_TIME_relative_multiply ( \
-    GNUNET_TIME_UNIT_MINUTES, 5)
+          GNUNET_TIME_UNIT_MINUTES, 5)
 
 /**
  * How long should clients cache a KYC success response?
  */
 #define EXPIRATION_KYC_SUCCESS GNUNET_TIME_relative_multiply ( \
-    GNUNET_TIME_UNIT_HOURS, 1)
+          GNUNET_TIME_UNIT_HOURS, 1)
 
 
 /**
@@ -447,7 +447,7 @@ ekr_finished (struct ExchangeKycRequest *ekr)
   if (NULL != kc->exchange_pending_head)
     return; /* wait for more */
   /* All exchange requests done, create final
-     big response from cummulated replies */
+     big response from cumulated replies */
   if ( (0 == json_array_size (kc->pending_kycs)) &&
        (0 == json_array_size (kc->timeout_kycs)) )
   {
