@@ -342,14 +342,12 @@ add_order (void *cls,
   {
     /* First try to find the order in the contracts */
     uint64_t os;
-    bool paid = false;
 
     qs = TMH_db->lookup_contract_terms (TMH_db->cls,
                                         po->instance_id,
                                         order_id,
                                         &contract_terms,
                                         &os,
-                                        &paid,
                                         NULL);
     if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
       GNUNET_break (os == order_serial);
@@ -809,6 +807,14 @@ TMH_private_get_orders (const struct TMH_RequestHandler *rh,
       }
     }
   }
+  po->of.session_id
+    = MHD_lookup_connection_value (connection,
+                                   MHD_GET_ARGUMENT_KIND,
+                                   "session_id");
+  po->of.fulfillment_url
+    = MHD_lookup_connection_value (connection,
+                                   MHD_GET_ARGUMENT_KIND,
+                                   "fulfillment_url");
   TALER_MHD_parse_request_timeout (connection,
                                    &po->long_poll_timeout);
   if (GNUNET_TIME_absolute_is_never (po->long_poll_timeout))
