@@ -49,7 +49,7 @@ transfer_cb (void *cls,
              const char *payto_uri,
              const char *exchange_url,
              uint64_t transfer_serial_id,
-             struct GNUNET_TIME_Absolute execution_time,
+             struct GNUNET_TIME_Timestamp execution_time,
              bool verified,
              bool confirmed)
 {
@@ -72,11 +72,11 @@ transfer_cb (void *cls,
     GNUNET_JSON_pack_bool ("confirmed",
                            confirmed),
     GNUNET_JSON_pack_allow_null (
-      GNUNET_JSON_pack_time_abs (
+      GNUNET_JSON_pack_timestamp (
         "execution_time",
-        GNUNET_TIME_absolute_is_never (execution_time)
-        ? GNUNET_TIME_UNIT_ZERO_ABS /* => field omitted */
-        : execution_time) ));
+        GNUNET_TIME_absolute_is_never (execution_time.abs_time)
+        ? GNUNET_TIME_UNIT_ZERO_TS /* => field omitted */
+         : execution_time)) );
   GNUNET_assert (0 ==
                  json_array_append_new (ja,
                                         r));
@@ -97,12 +97,13 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
                            struct TMH_HandlerContext *hc)
 {
   const char *payto_uri;
-  struct GNUNET_TIME_Absolute before = GNUNET_TIME_UNIT_FOREVER_ABS;
-  struct GNUNET_TIME_Absolute after = GNUNET_TIME_UNIT_ZERO_ABS;
+  struct GNUNET_TIME_Timestamp before = GNUNET_TIME_UNIT_FOREVER_TS;
+  struct GNUNET_TIME_Timestamp after = GNUNET_TIME_UNIT_ZERO_TS;
   int64_t limit = -20;
   uint64_t offset;
   enum TALER_EXCHANGE_YesNoAll verified;
 
+  (void) rh;
   payto_uri = MHD_lookup_connection_value (connection,
                                            MHD_GET_ARGUMENT_KIND,
                                            "payto_uri");
@@ -114,8 +115,8 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
                                             "before");
     if ( (NULL != before_s) &&
          (GNUNET_OK !=
-          GNUNET_STRINGS_fancy_time_to_absolute (before_s,
-                                                 &before)) )
+          GNUNET_STRINGS_fancy_time_to_timestamp (before_s,
+                                                  &before)) )
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_BAD_REQUEST,
                                          TALER_EC_GENERIC_PARAMETER_MALFORMED,
@@ -129,8 +130,8 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
                                            "after");
     if ( (NULL != after_s) &&
          (GNUNET_OK !=
-          GNUNET_STRINGS_fancy_time_to_absolute (after_s,
-                                                 &after)) )
+          GNUNET_STRINGS_fancy_time_to_timestamp (after_s,
+                                                  &after)) )
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_BAD_REQUEST,
                                          TALER_EC_GENERIC_PARAMETER_MALFORMED,
