@@ -28,12 +28,6 @@
 #include <taler/taler_json_lib.h>
 
 
-/**
- * How often do we retry the simple INSERT database transaction?
- */
-#define MAX_RETRIES 3
-
-
 /**
  * Determine the cause of the PATCH failure in more detail and report.
  *
@@ -109,17 +103,12 @@ TMH_private_patch_templates_ID (const struct TMH_RequestHandler *rh,
   const char *template_id = hc->infix;
   struct TALER_MERCHANTDB_TemplateDetails tp = {0};
   enum GNUNET_DB_QueryStatus qs;
-  uint32_t pos_algorithm;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("template_description",
                              (const char **) &tp.template_description),
     GNUNET_JSON_spec_mark_optional (
-      GNUNET_JSON_spec_uint32 ("pos_algorithm",
-                               &pos_algorithm),
-      NULL),
-    GNUNET_JSON_spec_mark_optional (
-      GNUNET_JSON_spec_string ("pos_key",
-                               (const char **) &tp.pos_key),
+      GNUNET_JSON_spec_string ("otp_id",
+                               (const char **) &tp.otp_id),
       NULL),
     GNUNET_JSON_spec_json ("template_contract",
                            &tp.template_contract),
@@ -140,7 +129,6 @@ TMH_private_patch_templates_ID (const struct TMH_RequestHandler *rh,
              : MHD_NO;
   }
 
-  tp.pos_algorithm = (enum TALER_MerchantConfirmationAlgorithm) pos_algorithm;
   if (! TMH_template_contract_valid (tp.template_contract))
   {
     GNUNET_break_op (0);
