@@ -32,7 +32,8 @@
  * and do not want to wake up too often.
  */
 #define BANK_TIMEOUT GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_MINUTES, \
-                                                    30)
+                                                    5)
+
 
 /**
  * Information about a watch job.
@@ -54,6 +55,11 @@ struct Watch
    */
   struct GNUNET_SCHEDULER_Task *task;
 
+  /**
+   * Dynamically adjusted long polling time-out.
+   */
+  struct GNUNET_TIME_Relative bank_timeout;
+
   /**
    * For which instance are we importing bank transfers?
    */
@@ -80,6 +86,16 @@ struct Watch
    */
   struct GNUNET_TIME_Relative delay;
 
+  /**
+   * When did we start our last HTTP request?
+   */
+  struct GNUNET_TIME_Absolute start_time;
+
+  /**
+   * How long should long-polling take at least?
+   */
+  struct GNUNET_TIME_Absolute long_poll_timeout;
+
   /**
    * Login data for the bank.
    */
@@ -144,6 +160,17 @@ static unsigned int batch_size = 32;
  */
 static int test_mode;
 
+/**
+ * #GNUNET_YES if we are in persistent mode and do
+ * not exit on #config_changed.
+ */
+static int persist_mode;
+
+/**
+ * Set to true if we are shutting down due to a
+ * configuration change.
+ */
+static bool config_changed_flag;
 
 /**
  * Save progress in DB.
@@ -315,7 +342,8 @@ credit_cb (
   {
   case 0:
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Invalid HTTP response from bank\n");
+                "Invalid HTTP response (HTTP status: 0, %d) from bank\n",
+                ec);
     w->delay = GNUNET_TIME_STD_BACKOFF (w->delay);
     break;
   case MHD_HTTP_OK:
@@ -323,6 +351,7 @@ credit_cb (
       enum GNUNET_DB_QueryStatus qs;
       char *exchange_url;
       struct TALER_WireTransferIdentifierRawP wtid;
+      char *credit_payto;
 
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Received wire transfer `%s' over %s\n",
@@ -341,35 +370,122 @@ credit_cb (
         w->start_row = serial_id;
         return GNUNET_OK;
       }
+      /* FIXME: consider grouping multiple inserts
+         into one bigger transaction with just one
+         notify! */
+      credit_payto = TALER_payto_normalize (details->credit_account_uri);
       qs = db_plugin->insert_transfer (db_plugin->cls,
                                        w->instance_id,
                                        exchange_url,
                                        &wtid,
                                        &details->amount,
-                                       details->credit_account_uri,
+                                       credit_payto,
                                        true /* confirmed */);
-      if (0 == qs)
+      if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
       {
-        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                    "Inserting transfer for %s into database failed. Is the credit account %s configured correctly?\n",
-                    w->instance_id,
-                    details->credit_account_uri);
+        struct TALER_Amount total;
+        struct TALER_Amount wfee;
+        struct TALER_Amount eamount;
+        struct GNUNET_TIME_Timestamp timestamp;
+        bool have_esig;
+        bool verified;
+
+        qs = db_plugin->lookup_transfer (db_plugin->cls,
+                                         w->instance_id,
+                                         exchange_url,
+                                         &wtid,
+                                         &total,
+                                         &wfee,
+                                         &eamount,
+                                         &timestamp,
+                                         &have_esig,
+                                         &verified);
+        if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                      "Inserting transfer for %s into database failed. Is the credit account %s configured correctly?\n",
+                      w->instance_id,
+                      credit_payto);
+        }
+        if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
+        {
+          if (0 !=
+              TALER_amount_cmp (&total,
+                                &details->amount))
+          {
+            GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                        "Inserting transfer for %s into database failed. An entry exists for a different transfer amount (%s)!\n",
+                        w->instance_id,
+                        TALER_amount2s (&total));
+          }
+          else
+          {
+            GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                        "Inserting transfer for %s into database failed. An equivalent entry already exists.\n",
+                        w->instance_id);
+          }
+        }
       }
+      GNUNET_free (credit_payto);
       GNUNET_free (exchange_url);
       if (qs < 0)
       {
         GNUNET_break (0);
         GNUNET_SCHEDULER_shutdown ();
+        w->hh = NULL;
         return GNUNET_SYSERR;
       }
+      /* Success => reset back-off timer! */
+      w->delay = GNUNET_TIME_UNIT_ZERO;
+      {
+        struct GNUNET_DB_EventHeaderP es = {
+          .size = htons (sizeof (es)),
+          .type = htons (TALER_DBEVENT_MERCHANT_WIRE_TRANSFER_CONFIRMED)
+        };
+
+        db_plugin->event_notify (db_plugin->cls,
+                                 &es,
+                                 NULL,
+                                 0);
+      }
     }
     w->start_row = serial_id;
     return GNUNET_OK;
   case MHD_HTTP_NO_CONTENT:
     save (w);
-    w->delay = GNUNET_TIME_UNIT_ZERO;
+    /* Delay artificially if server returned before long-poll timeout */
+    if (! w->found)
+      w->delay = GNUNET_TIME_absolute_get_remaining (w->long_poll_timeout);
+    break;
+  case MHD_HTTP_NOT_FOUND:
+    /* configuration likely wrong, wait at least 1 minute, backoff up to 15 minutes! */
+    w->delay = GNUNET_TIME_relative_max (GNUNET_TIME_UNIT_MINUTES,
+                                         GNUNET_TIME_STD_BACKOFF (w->delay));
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Bank claims account is unknown, waiting for %s before trying again\n",
+                GNUNET_TIME_relative2s (w->delay,
+                                        true));
+    break;
+  case MHD_HTTP_GATEWAY_TIMEOUT:
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Gateway timeout, adjusting long polling threshold\n");
+    /* Limit new timeout at request delay */
+    w->bank_timeout
+      = GNUNET_TIME_relative_min (GNUNET_TIME_absolute_get_duration (
+                                    w->start_time),
+                                  w->bank_timeout);
+    /* set the timeout a bit earlier */
+    w->bank_timeout
+      = GNUNET_TIME_relative_subtract (w->bank_timeout,
+                                       GNUNET_TIME_UNIT_SECONDS);
+    /* do not allow it to go to zero */
+    w->bank_timeout
+      = GNUNET_TIME_relative_max (w->bank_timeout,
+                                  GNUNET_TIME_UNIT_SECONDS);
+    w->delay = GNUNET_TIME_STD_BACKOFF (w->delay);
     break;
   default:
+    /* Something went wrong, try again, but with back-off */
     w->delay = GNUNET_TIME_STD_BACKOFF (w->delay);
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Unexpected HTTP status code %u(%d) from bank\n",
@@ -401,13 +517,17 @@ do_work (void *cls)
 
   w->task = NULL;
   w->found = false;
+  w->long_poll_timeout
+    = GNUNET_TIME_relative_to_absolute (w->bank_timeout);
+  w->start_time
+    = GNUNET_TIME_absolute_get ();
   w->hh = TALER_MERCHANT_BANK_credit_history (ctx,
                                               &w->ad,
                                               w->start_row,
                                               batch_size,
                                               test_mode
                                               ? GNUNET_TIME_UNIT_ZERO
-                                              : BANK_TIMEOUT,
+                                              : w->bank_timeout,
                                               &credit_cb,
                                               w);
   if (NULL == w->hh)
@@ -442,6 +562,7 @@ start_watch (
   struct Watch *w = GNUNET_new (struct Watch);
 
   (void) cls;
+  w->bank_timeout = BANK_TIMEOUT;
   if (GNUNET_OK !=
       TALER_MERCHANT_BANK_auth_parse_json (credit_facade_credentials,
                                            credit_facade_url,
@@ -453,7 +574,7 @@ start_watch (
                 payto_uri);
     GNUNET_free (w);
     GNUNET_SCHEDULER_shutdown ();
-    global_ret = 1;
+    global_ret = EXIT_NOTCONFIGURED;
     return;
   }
 
@@ -461,7 +582,7 @@ start_watch (
                                w_tail,
                                w);
   w->instance_id = GNUNET_strdup (instance);
-  w->payto_uri = GNUNET_strdup (payto_uri);
+  w->payto_uri = TALER_payto_normalize (payto_uri);
   w->start_row = last_serial;
   w->task = GNUNET_SCHEDULER_add_now (&do_work,
                                       w);
@@ -484,6 +605,12 @@ config_changed (void *cls,
   (void) cls;
   (void) extra;
   (void) extra_size;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Configuration changed, %s\n",
+              0 == persist_mode
+              ? "restarting"
+              : "reinitializing");
+  config_changed_flag = true;
   GNUNET_SCHEDULER_shutdown ();
 }
 
@@ -515,6 +642,7 @@ run (void *cls,
   {
     GNUNET_break (0);
     GNUNET_SCHEDULER_shutdown ();
+    global_ret = EXIT_NO_RESTART;
     return;
   }
   if (NULL ==
@@ -523,6 +651,7 @@ run (void *cls,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to initialize DB subsystem\n");
     GNUNET_SCHEDULER_shutdown ();
+    global_ret = EXIT_NOTCONFIGURED;
     return;
   }
   if (GNUNET_OK !=
@@ -531,6 +660,7 @@ run (void *cls,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to connect to database\n");
     GNUNET_SCHEDULER_shutdown ();
+    global_ret = EXIT_NO_RESTART;
     return;
   }
   {
@@ -556,14 +686,16 @@ run (void *cls,
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Failed to obtain wirewatch accounts from database\n");
       GNUNET_SCHEDULER_shutdown ();
+      global_ret = EXIT_NO_RESTART;
       return;
     }
-    if ( (0 == qs) &&
+    if ( (NULL == w_head) &&
          (GNUNET_YES == test_mode) )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "No active wirewatch accounts in database and in test mode. Exiting.\n");
       GNUNET_SCHEDULER_shutdown ();
+      global_ret = EXIT_SUCCESS;
       return;
     }
   }
@@ -582,6 +714,12 @@ main (int argc,
       char *const *argv)
 {
   struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_flag ('p',
+                               "persist",
+                               "run in persist mode and do not exit on configuration changes",
+                               &persist_mode),
+    GNUNET_GETOPT_option_timetravel ('T',
+                                     "timetravel"),
     GNUNET_GETOPT_option_flag ('t',
                                "test",
                                "run in test mode and exit when idle",
@@ -596,13 +734,17 @@ main (int argc,
                                     &argc, &argv))
     return EXIT_INVALIDARGUMENT;
   TALER_OS_init ();
-  ret = GNUNET_PROGRAM_run (
-    argc, argv,
-    "taler-merchant-wirewatch",
-    gettext_noop (
-      "background process that watches for incoming wire transfers to the merchant bank account"),
-    options,
-    &run, NULL);
+  do {
+    config_changed_flag = false;
+    ret = GNUNET_PROGRAM_run (
+      argc, argv,
+      "taler-merchant-wirewatch",
+      gettext_noop (
+        "background process that watches for incoming wire transfers to the merchant bank account"),
+      options,
+      &run, NULL);
+  } while ( (1 == persist_mode) &&
+            config_changed_flag);
   GNUNET_free_nz ((void *) argv);
   if (GNUNET_SYSERR == ret)
     return EXIT_INVALIDARGUMENT;
