@@ -69,5 +69,15 @@ TALER_MERCHANTDB_pending_webhook_details_free (
   GNUNET_free (pwb->body);
 }
 
+void
+TALER_MERCHANTDB_token_family_details_free (
+  struct TALER_MERCHANTDB_TokenFamilyDetails *tf)
+{
+  GNUNET_free (tf->slug);
+  GNUNET_free (tf->name);
+  GNUNET_free (tf->description);
+  json_decref (tf->description_i18n);
+}
+
 
 /* end of merchantdb_helper.c */
