@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014--2023 Taler Systems SA
+  (C) 2014--2024 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -43,13 +43,13 @@
 #include "pg_update_account.h"
 #include "pg_lookup_instances.h"
 #include "pg_lookup_transfers.h"
+#include "pg_lookup_pending_deposits.h"
 #include "pg_update_wirewatch_progress.h"
 #include "pg_select_wirewatch_accounts.h"
 #include "pg_select_open_transfers.h"
 #include "pg_delete_exchange_accounts.h"
 #include "pg_select_accounts_by_exchange.h"
 #include "pg_insert_exchange_account.h"
-#include "pg_lookup_reserves.h"
 #include "pg_lookup_instance_auth.h"
 #include "pg_lookup_otp_devices.h"
 #include "pg_update_transfer_status.h"
@@ -76,6 +76,7 @@
 #include "pg_insert_order.h"
 #include "pg_unlock_inventory.h"
 #include "pg_insert_order_lock.h"
+#include "pg_lookup_contract_terms3.h"
 #include "pg_lookup_contract_terms2.h"
 #include "pg_lookup_contract_terms.h"
 #include "pg_insert_contract_terms.h"
@@ -96,7 +97,6 @@
 #include "pg_refund_coin.h"
 #include "pg_lookup_order_status.h"
 #include "pg_lookup_order_status_by_serial.h"
-#include "pg_lookup_payment_status.h"
 #include "pg_lookup_deposits_by_order.h"
 #include "pg_lookup_transfer_details_by_order.h"
 #include "pg_mark_order_wired.h"
@@ -112,15 +112,6 @@
 #include "pg_lookup_transfer.h"
 #include "pg_lookup_transfer_summary.h"
 #include "pg_lookup_transfer_details.h"
-#include "pg_lookup_pending_reserves.h"
-#include "pg_lookup_reserve.h"
-#include "pg_delete_reserve.h"
-#include "pg_purge_reserve.h"
-#include "pg_lookup_pickup.h"
-#include "pg_lookup_reward.h"
-#include "pg_lookup_rewards.h"
-#include "pg_lookup_reward_details.h"
-#include "pg_insert_pickup_blind_signature.h"
 #include "pg_lookup_webhooks.h"
 #include "pg_lookup_webhook.h"
 #include "pg_delete_webhook.h"
@@ -131,8 +122,7 @@
 #include "pg_insert_pending_webhook.h"
 #include "pg_update_pending_webhook.h"
 #include "pg_lookup_pending_webhooks.h"
-// ^^^^^ + lookup_future_webhook
-// ^^^^^ + lookup_all_webhooks
+#include "pg_update_deposit_confirmation_status.h"
 #include "pg_set_transfer_status_to_confirmed.h"
 #include "pg_insert_exchange_keys.h"
 #include "pg_select_exchange_keys.h"
@@ -143,10 +133,11 @@
 #include "pg_insert_transfer.h"
 #include "pg_insert_transfer_details.h"
 #include "pg_store_wire_fee_by_exchange.h"
-#include "pg_insert_reserve.h"
-#include "pg_activate_reserve.h"
-#include "pg_authorize_reward.h"
-#include "pg_insert_pickup.h"
+#include "pg_insert_token_family.h"
+#include "pg_lookup_token_family.h"
+#include "pg_lookup_token_families.h"
+#include "pg_delete_token_family.h"
+#include "pg_update_token_family.h"
 
 
 /**
@@ -197,6 +188,7 @@ postgres_create_tables (void *cls)
     GNUNET_PQ_make_try_execute ("SET search_path TO merchant;"),
     GNUNET_PQ_EXECUTE_STATEMENT_END
   };
+  enum GNUNET_GenericReturnValue ret;
 
   conn = GNUNET_PQ_connect_with_cfg (pc->cfg,
                                      "merchantdb-postgres",
@@ -205,8 +197,10 @@ postgres_create_tables (void *cls)
                                      NULL);
   if (NULL == conn)
     return GNUNET_SYSERR;
+  ret = GNUNET_PQ_exec_sql (conn,
+                            "procedures");
   GNUNET_PQ_disconnect (conn);
-  return GNUNET_OK;
+  return ret;
 }
 
 
@@ -217,7 +211,7 @@ postgres_create_tables (void *cls)
  * @param es specification of the event to listen for
  * @param timeout how long to wait for the event
  * @param cb function to call when the event happens, possibly
- *         mulrewardle times (until cancel is invoked)
+ *         multiple times (until cancel is invoked)
  * @param cb_cls closure for @a cb
  * @return handle useful to cancel the listener
  */
@@ -295,21 +289,17 @@ check_connection (struct PostgresClosure *pg)
   GNUNET_PQ_reconnect_if_down (pg->conn);
 }
 
+
 /**
  * Establish connection to the database.
  *
  * @param cls plugin context
  * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure
  */
-static int
+static enum GNUNET_GenericReturnValue
 postgres_connect (void *cls)
 {
   struct PostgresClosure *pg = cls;
-  struct GNUNET_PQ_PreparedStatement ps[] = {
-    GNUNET_PQ_make_prepare ("end_transaction",
-                            "COMMIT"),
-    GNUNET_PQ_PREPARED_STATEMENT_END
-  };
   struct GNUNET_PQ_ExecuteStatement es[] = {
     GNUNET_PQ_make_try_execute ("SET search_path TO merchant;"),
     GNUNET_PQ_EXECUTE_STATEMENT_END
@@ -319,7 +309,7 @@ postgres_connect (void *cls)
                                          "merchantdb-postgres",
                                          NULL,
                                          es,
-                                         ps);
+                                         NULL);
   pg->prep_gen++;
   if (NULL == pg->conn)
     return GNUNET_SYSERR;
@@ -455,6 +445,8 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_lookup_contract_terms;
   plugin->lookup_contract_terms2
     = &TMH_PG_lookup_contract_terms2;
+  plugin->lookup_contract_terms3
+    = &TMH_PG_lookup_contract_terms3;
   plugin->insert_contract_terms
     = &TMH_PG_insert_contract_terms;
   plugin->update_contract_terms
@@ -479,8 +471,6 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_lookup_order_status;
   plugin->lookup_order_status_by_serial
     = &TMH_PG_lookup_order_status_by_serial;
-  plugin->lookup_payment_status
-    = &TMH_PG_lookup_payment_status;
   plugin->lookup_deposits_by_order
     = &TMH_PG_lookup_deposits_by_order;
   plugin->lookup_transfer_details_by_order
@@ -529,26 +519,6 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_select_account;
   plugin->select_accounts
     = &TMH_PG_select_accounts;
-  plugin->lookup_reserves
-    = &TMH_PG_lookup_reserves;
-  plugin->lookup_pending_reserves
-    = &TMH_PG_lookup_pending_reserves;
-  plugin->lookup_reserve
-    = &TMH_PG_lookup_reserve;
-  plugin->delete_reserve
-    = &TMH_PG_delete_reserve;
-  plugin->purge_reserve
-    = &TMH_PG_purge_reserve;
-  plugin->lookup_pickup
-    = &TMH_PG_lookup_pickup;
-  plugin->lookup_reward
-    = &TMH_PG_lookup_reward;
-  plugin->lookup_rewards
-    = &TMH_PG_lookup_rewards;
-  plugin->lookup_reward_details
-    = &TMH_PG_lookup_reward_details;
-  plugin->insert_pickup_blind_signature
-    = &TMH_PG_insert_pickup_blind_signature;
   plugin->select_open_transfers
     = &TMH_PG_select_open_transfers;
   plugin->insert_exchange_keys
@@ -563,14 +533,6 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_insert_transfer_details;
   plugin->store_wire_fee_by_exchange
     = &TMH_PG_store_wire_fee_by_exchange;
-  plugin->insert_reserve
-    = &TMH_PG_insert_reserve;
-  plugin->activate_reserve
-    = &TMH_PG_activate_reserve;
-  plugin->authorize_reward
-    = &TMH_PG_authorize_reward;
-  plugin->insert_pickup
-    = &TMH_PG_insert_pickup;
   plugin->lookup_webhooks
     = &TMH_PG_lookup_webhooks;
   plugin->lookup_webhook
@@ -581,6 +543,8 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_insert_webhook;
   plugin->update_webhook
     = &TMH_PG_update_webhook;
+  plugin->lookup_pending_deposits
+    = &TMH_PG_lookup_pending_deposits;
   plugin->lookup_webhook_by_event
     = &TMH_PG_lookup_webhook_by_event;
   plugin->lookup_all_webhooks
@@ -601,6 +565,19 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_select_accounts_by_exchange;
   plugin->insert_exchange_account
     = &TMH_PG_insert_exchange_account;
+  plugin->insert_token_family
+    = &TMH_PG_insert_token_family;
+  plugin->lookup_token_family
+    = &TMH_PG_lookup_token_family;
+  plugin->lookup_token_families
+    = &TMH_PG_lookup_token_families;
+  plugin->delete_token_family
+    = &TMH_PG_delete_token_family;
+  plugin->update_token_family
+    = &TMH_PG_update_token_family;
+  plugin->update_deposit_confirmation_status
+    = &TMH_PG_update_deposit_confirmation_status;
+
   return plugin;
 }
 
