@@ -573,6 +573,35 @@ TALER_TESTING_cmd_merchant_post_orders2 (
   const char *duplicate_of);
 
 
+/**
+ * Create an order with a specific fulfillment URL.
+ * Does not claim the order.
+ *
+ * @param label command label
+ * @param cfg configuration to use
+ * @param merchant_url base URL of the merchant serving
+ *        the proposal request
+ * @param http_status expected HTTP status
+ * @param order_id ID of the order to create
+ * @param refund_deadline the deadline for refunds on this order
+ * @param pay_deadline the deadline for payment on this order
+ * @param fulfillment_url the fulfillment URL to use
+ * @param amount the amount this order is for
+ * @return the command
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_orders3 (
+  const char *label,
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *order_id,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
+  const char *fulfillment_url,
+  const char *amount);
+
+
 /**
  * Define a "GET /orders" CMD.
  *
@@ -581,7 +610,7 @@ TALER_TESTING_cmd_merchant_post_orders2 (
  *        GET /orders request.
  * @param http_status expected HTTP response code.
  * @param ... NULL-terminated list of labels (const char *) of
- *        reserve (commands) we expect to be returned in the list
+ *        order (commands) we expect to be returned in the list
  *        (assuming @a http_code is #MHD_HTTP_OK)
  * @return the command.
  */
@@ -616,9 +645,10 @@ TALER_TESTING_cmd_poll_orders_start (const char *label,
  * @param poll_start_reference reference to the #TALER_TESTING_cmd_poll_orders_start command
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_poll_orders_conclude (const char *label,
-                                        unsigned int http_status,
-                                        const char *poll_start_reference);
+TALER_TESTING_cmd_poll_orders_conclude (
+  const char *label,
+  unsigned int http_status,
+  const char *poll_start_reference);
 
 
 /**
@@ -634,13 +664,41 @@ TALER_TESTING_cmd_poll_orders_conclude (const char *label,
  * @param http_status expected HTTP response code for the request.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_wallet_get_order (const char *label,
-                                    const char *merchant_url,
-                                    const char *order_reference,
-                                    bool paid,
-                                    bool refunded,
-                                    bool refund_pending,
-                                    unsigned int http_status);
+TALER_TESTING_cmd_wallet_get_order (
+  const char *label,
+  const char *merchant_url,
+  const char *order_reference,
+  bool paid,
+  bool refunded,
+  bool refund_pending,
+  unsigned int http_status);
+
+
+/**
+ * Define a GET /orders/$ORDER_ID CMD.
+ *
+ * @param label the command label
+ * @param merchant_url base URL of the merchant which will
+ *        serve the request.
+ * @param order_reference reference to a command that created an order.
+ * @param session_id session ID to check for
+ * @param paid whether the order has been paid for or not.
+ * @param refunded whether the order has been refunded.
+ * @param refund_pending whether the order has refunds that haven't been obtained.
+ * @param repurchase_order_ref command of a paid equivalent order the merchant should be referring us to, or NULL
+ * @param http_status expected HTTP response code for the request.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_wallet_get_order2 (
+  const char *label,
+  const char *merchant_url,
+  const char *order_reference,
+  const char *session_id,
+  bool paid,
+  bool refunded,
+  bool refund_pending,
+  const char *repurchase_order_ref,
+  unsigned int http_status);
 
 
 /**
@@ -737,13 +795,14 @@ TALER_TESTING_cmd_wallet_poll_order_conclude2 (
  *        this parameter is ignored.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_get_order (const char *label,
-                                      const char *merchant_url,
-                                      const char *order_reference,
-                                      enum TALER_MERCHANT_OrderStatusCode osc,
-                                      bool refunded,
-                                      unsigned int http_status,
-                                      ...);
+TALER_TESTING_cmd_merchant_get_order (
+  const char *label,
+  const char *merchant_url,
+  const char *order_reference,
+  enum TALER_MERCHANT_OrderStatusCode osc,
+  bool refunded,
+  unsigned int http_status,
+  ...);
 
 
 /**
@@ -770,16 +829,40 @@ TALER_TESTING_cmd_merchant_get_order (const char *label,
  * @param http_status expected HTTP response code for the request.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_get_order2 (const char *label,
-                                       const char *merchant_url,
-                                       const char *order_reference,
-                                       enum TALER_MERCHANT_OrderStatusCode osc,
-                                       bool wired,
-                                       const char **transfers,
-                                       bool refunded,
-                                       const char **refunds,
-                                       const char **forgets,
-                                       unsigned int http_status);
+TALER_TESTING_cmd_merchant_get_order2 (
+  const char *label,
+  const char *merchant_url,
+  const char *order_reference,
+  enum TALER_MERCHANT_OrderStatusCode osc,
+  bool wired,
+  const char **transfers,
+  bool refunded,
+  const char **refunds,
+  const char **forgets,
+  unsigned int http_status);
+
+
+/**
+ * Define a GET /private/orders/$ORDER_ID CMD.
+ *
+ * @param label the command label
+ * @param merchant_url base URL of the merchant which will
+ *        serve the request.
+ * @param order_reference reference to a command that created an order.
+ * @param osc expected order status
+ * @param session_id session ID the payment must be bound to
+ * @param repurchase_order_ref command of a paid equivalent order the merchant should be referring us to, or NULL
+ * @param expected_http_status expected HTTP response code for the request.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_order3 (
+  const char *label,
+  const char *merchant_url,
+  const char *order_reference,
+  enum TALER_MERCHANT_OrderStatusCode osc,
+  const char *session_id,
+  const char *repurchase_order_ref,
+  unsigned int expected_http_status);
 
 
 /**
@@ -836,14 +919,15 @@ TALER_TESTING_cmd_merchant_claim_order (const char *label,
  * @return the command
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_pay_order (const char *label,
-                                      const char *merchant_url,
-                                      unsigned int http_status,
-                                      const char *proposal_reference,
-                                      const char *coin_reference,
-                                      const char *amount_with_fee,
-                                      const char *amount_without_fee,
-                                      const char *session_id);
+TALER_TESTING_cmd_merchant_pay_order (
+  const char *label,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *proposal_reference,
+  const char *coin_reference,
+  const char *amount_with_fee,
+  const char *amount_without_fee,
+  const char *session_id);
 
 
 /**
@@ -857,11 +941,12 @@ TALER_TESTING_cmd_merchant_pay_order (const char *label,
  * @return the command
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_post_orders_paid (const char *label,
-                                             const char *merchant_url,
-                                             const char *pay_reference,
-                                             const char *session_id,
-                                             unsigned int http_status);
+TALER_TESTING_cmd_merchant_post_orders_paid (
+  const char *label,
+  const char *merchant_url,
+  const char *pay_reference,
+  const char *session_id,
+  unsigned int http_status);
 
 
 /**
@@ -874,10 +959,11 @@ TALER_TESTING_cmd_merchant_post_orders_paid (const char *label,
  * @return the command
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_order_abort (const char *label,
-                                        const char *merchant_url,
-                                        const char *pay_reference,
-                                        unsigned int http_status);
+TALER_TESTING_cmd_merchant_order_abort (
+  const char *label,
+  const char *merchant_url,
+  const char *pay_reference,
+  unsigned int http_status);
 
 
 /**
@@ -916,12 +1002,13 @@ TALER_TESTING_cmd_merchant_forget_order (
  * @return the command.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_order_refund (const char *label,
-                                         const char *merchant_url,
-                                         const char *reason,
-                                         const char *order_id,
-                                         const char *refund_amount,
-                                         unsigned int http_code);
+TALER_TESTING_cmd_merchant_order_refund (
+  const char *label,
+  const char *merchant_url,
+  const char *reason,
+  const char *order_id,
+  const char *refund_amount,
+  unsigned int http_code);
 
 
 /**
@@ -939,11 +1026,12 @@ TALER_TESTING_cmd_merchant_order_refund (const char *label,
  * @return the command.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_wallet_order_refund (const char *label,
-                                       const char *merchant_url,
-                                       const char *order_ref,
-                                       unsigned int http_code,
-                                       ...);
+TALER_TESTING_cmd_wallet_order_refund (
+  const char *label,
+  const char *merchant_url,
+  const char *order_ref,
+  unsigned int http_code,
+  ...);
 
 
 /**
@@ -957,10 +1045,11 @@ TALER_TESTING_cmd_wallet_order_refund (const char *label,
  * @return the command.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_delete_order (const char *label,
-                                         const char *merchant_url,
-                                         const char *order_id,
-                                         unsigned int http_status);
+TALER_TESTING_cmd_merchant_delete_order (
+  const char *label,
+  const char *merchant_url,
+  const char *order_id,
+  unsigned int http_status);
 
 
 /* ******************* /transfers *************** */
@@ -1071,403 +1160,6 @@ TALER_TESTING_cmd_merchant_delete_transfer (const char *label,
                                             unsigned int http_status);
 
 
-/* ******************* /reserves *************** */
-
-
-/**
- * Define a "POST /reserves" CMD
- *
- * @param label command label.
- * @param merchant_url url to the murchant.
- * @param initial_balance initial amount in the reserve.
- * @param exchange_url url to the exchange
- * @param wire_method wire transfer method to use for this reserve
- * @param http_status expected HTTP response code.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_post_reserves (const char *label,
-                                          const char *merchant_url,
-                                          const char *initial_balance,
-                                          const char *exchange_url,
-                                          const char *wire_method,
-                                          unsigned int http_status);
-
-
-/**
- * This commands does not query the backend at all,
- * but just makes up a fake reserve.
- *
- * @param label command label.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_post_reserves_fake (const char *label);
-
-
-/**
- * Define a "GET reserve" CMD.
- *
- * @param label command label.
- * @param merchant_url base URL of the merchant serving the request.
- * @param http_status expected HTTP response code.
- * @param reserve_reference reference to a "POST /reserves" that provides the
- *        information we are expecting.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_get_reserve (const char *label,
-                                        const char *merchant_url,
-                                        unsigned int http_status,
-                                        const char *reserve_reference);
-
-
-/**
- * Define a "GET reserve" CMD.
- *
- * @param label command label.
- * @param merchant_url base URL of the merchant serving the request.
- * @param http_status expected HTTP response code.
- * @param reserve_reference reference to a "POST /reserves" that provides the
- *        information we are expecting.
- * @param ... NULL-terminated list of labels (const char *) of
- *        reward (commands) we expect to be returned in the list
- *        (assuming @a http_code is #MHD_HTTP_OK)
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_get_reserve_with_rewards (const char *label,
-                                                     const char *merchant_url,
-                                                     unsigned int http_status,
-                                                     const char *
-                                                     reserve_reference,
-                                                     ...);
-
-
-/**
- * Define a "GET /reserves" CMD
- *
- * @param label command label.
- * @param merchant_url url to the merchant.
- * @param http_status expected HTTP response code.
- * @param ... NULL-terminated list of labels (const char *) of
- *        reserve (commands) we expect to be returned in the list
- *        (assuming @a http_code is #MHD_HTTP_OK)
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_get_reserves (const char *label,
-                                         const char *merchant_url,
-                                         unsigned int http_status,
-                                         ...);
-
-
-/**
- * Define a "DELETE reserve" CMD.
- *
- * @param label command label.
- * @param merchant_url base URL of the merchant serving the
- *        DELETE /reserves/$RESERVE_PUB request.
- * @param reserve_reference command label of a command providing a reserve
- * @param http_status expected HTTP response code.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_delete_reserve (const char *label,
-                                           const char *merchant_url,
-                                           const char *reserve_reference,
-                                           unsigned int http_status);
-
-
-/**
- * Define a "PURGE reserve" CMD.
- *
- * @param label command label.
- * @param merchant_url base URL of the merchant serving the
- *        DELETE /reserves/$RESERVE_PUB request.
- * @param reserve_reference command label of a command providing a reserve
- * @param http_status expected HTTP response code.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_purge_reserve (const char *label,
-                                          const char *merchant_url,
-                                          const char *reserve_reference,
-                                          unsigned int http_status);
-
-
-/**
- * Define a get rewards CMD.
- *
- * @param label the command label
- * @param merchant_url base URL of the merchant which will
- *        server the /reward-query request.
- * @param http_status expected HTTP response code for the
- *        /reward-query request.
- * @param ... NULL-terminated list of labels (const char *) of
- *        reward (commands) we expect to be returned in the list
- *        (assuming @a http_code is #MHD_HTTP_OK)
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_get_rewards (const char *label,
-                               const char *merchant_url,
-                               unsigned int http_status,
-                               ...);
-
-
-/**
- * Define a get rewards CMD.
- *
- * @param label the command label
- * @param merchant_url base URL of the merchant which will
- *        server the /reward-query request.
- * @param http_status expected HTTP response code for the
- *        /reward-query request.
- * @param offset row to start querying the database from.
- * @param limit how many rows to return (with direction).
- * @param ... NULL-terminated list of labels (const char *) of
- *        reward (commands) we expect to be returned in the list
- *        (assuming @a http_code is #MHD_HTTP_OK)
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_get_rewards2 (const char *label,
-                                const char *merchant_url,
-                                uint64_t offset,
-                                int64_t limit,
-                                unsigned int http_status,
-                                ...);
-
-
-/**
- * Define a GET /private/rewards/$REWARD_ID CMD.
- *
- * @param label the command label
- * @param merchant_url base URL of the merchant which will
- *        serve the request.
- * @param reward_reference reference to a command that created a reward.
- * @param http_status expected HTTP response code for the request.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_get_reward (const char *label,
-                                       const char *merchant_url,
-                                       const char *reward_reference,
-                                       unsigned int http_status);
-
-
-/**
- * Define a GET /private/rewards/$REWARD_ID CMD.
- *
- * @param label the command label
- * @param merchant_url base URL of the merchant which will
- *        serve the request.
- * @param reward_reference reference to a command that created a reward.
- * @param http_status expected HTTP response code for the request.
- * @param ... NULL-terminated list of labels (const char *) of
- *        pickup (commands) we expect to be returned in the list
- *        (assuming @a http_code is #MHD_HTTP_OK)
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_get_reward_with_pickups (const char *label,
-                                                    const char *merchant_url,
-                                                    const char *reward_reference,
-                                                    unsigned int http_status,
-                                                    ...);
-
-/**
- * Define a GET /rewards/$REWARD_ID CMD.
- *
- * @param label the command label
- * @param merchant_url base URL of the merchant which will
- *        serve the request.
- * @param reward_reference reference to a command that created a reward.
- * @param http_status expected HTTP response code for the request.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_wallet_get_reward (const char *label,
-                                     const char *merchant_url,
-                                     const char *reward_reference,
-                                     unsigned int http_status);
-
-
-/**
- * Define a GET /rewards/$REWARD_ID CMD.
- *
- * @param label the command label
- * @param merchant_url base URL of the merchant which will
- *        serve the request.
- * @param reward_reference reference to a command that created a reward.
- * @param amount_remaining the balance remaining after pickups.
- * @param http_status expected HTTP response code for the request.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_wallet_get_reward2 (const char *label,
-                                      const char *merchant_url,
-                                      const char *reward_reference,
-                                      const char *amount_remaining,
-                                      unsigned int http_status);
-
-
-/**
- * Create a /reward-authorize CMD.
- *
- * @param label this command label
- * @param merchant_url the base URL of the merchant that will
- *        serve the /reward-authorize request.
- * @param exchange_url the base URL of the exchange that owns
- *        the reserve from which the reward is going to be gotten.
- * @param http_status the HTTP response code which is expected
- *        for this operation.
- * @param justification human-readable justification for this
- *        reward authorization.
- * @param amount the amount to authorize for rewardping.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_reward_authorize (const char *label,
-                                    const char *merchant_url,
-                                    const char *exchange_url,
-                                    unsigned int http_status,
-                                    const char *justification,
-                                    const char *amount);
-
-
-/**
- * Create a /reward-authorize CMD.
- *
- * @param label this command label
- * @param merchant_url the base URL of the merchant that will
- *        serve the /reward-authorize request.
- * @param exchange_url the base URL of the exchange that owns
- *        the reserve from which the reward is going to be gotten.
- * @param reserve_reference reference to a command that created
- *        a reserve.
- * @param http_status the HTTP response code which is expected
- *        for this operation.
- * @param justification human-readable justification for this
- *        reward authorization.
- * @param amount the amount to authorize for rewardping.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_reward_authorize_from_reserve (const char *label,
-                                                 const char *merchant_url,
-                                                 const char *exchange_url,
-                                                 const char *reserve_reference,
-                                                 unsigned int http_status,
-                                                 const char *justification,
-                                                 const char *amount);
-
-
-/**
- * Create a /reward-authorize CMD, specifying the Taler error code
- * that is expected to be returned by the backend.
- *
- * @param label this command label
- * @param merchant_url the base URL of the merchant that will
- *        serve the /reward-authorize request.
- * @param exchange_url the base URL of the exchange that owns
- *        the reserve from which the reward is going to be gotten.
- * @param http_status the HTTP response code which is expected
- *        for this operation.
- * @param justification human-readable justification for this
- *        reward authorization.
- * @param amount the amount to authorize for rewardping.
- * @param ec expected Taler-defined error code.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_reward_authorize_with_ec (const char *label,
-                                            const char *merchant_url,
-                                            const char *exchange_url,
-                                            unsigned int http_status,
-                                            const char *justification,
-                                            const char *amount,
-                                            enum TALER_ErrorCode ec);
-
-
-/**
- * Create a /reward-authorize CMD, specifying the Taler error code
- * that is expected to be returned by the backend.
- *
- * @param label this command label
- * @param merchant_url the base URL of the merchant that will
- *        serve the /reward-authorize request.
- * @param exchange_url the base URL of the exchange that owns
- *        the reserve from which the reward is going to be gotten.
- * @param reserve_reference reference to a command that created
- *        a reserve.
- * @param http_status the HTTP response code which is expected
- *        for this operation.
- * @param justification human-readable justification for this
- *        reward authorization.
- * @param amount the amount to authorize for rewardping.
- * @param ec expected Taler-defined error code.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_reward_authorize_from_reserve_with_ec (
-  const char *label,
-  const char *merchant_url,
-  const char *exchange_url,
-  const char *reserve_reference,
-  unsigned int http_status,
-  const char *justification,
-  const char *amount,
-  enum TALER_ErrorCode ec);
-
-
-/**
- * This commands does not query the backend at all,
- * but just makes up a fake authorization id that will
- * be subsequently used by the "pick up" CMD in order
- * to test against such a case.
- *
- * @param label command label.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_reward_authorize_fake (const char *label);
-
-
-/**
- * Define a /reward-pickup CMD, equipped with the expected error
- * code.
- *
- * @param label the command label
- * @param merchant_url base URL of the backend which will serve
- *        the /reward-pickup request.
- * @param http_status expected HTTP response code.
- * @param authorize_reference reference to a /reward-autorize CMD
- *        that offers a reward id to pick up.
- * @param amounts array of string-defined amounts that specifies
- *        which denominations will be accepted for rewardping.
- * @param ec expected Taler error code.
- * @return the command
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_reward_pickup_with_ec (const char *label,
-                                         const char *merchant_url,
-                                         unsigned int http_status,
-                                         const char *authorize_reference,
-                                         const char **amounts,
-                                         enum TALER_ErrorCode ec);
-
-/**
- * Define a /reward-pickup CMD.
- *
- * @param label the command label
- * @param merchant_url base URL of the backend which will serve
- *        the /reward-pickup request.
- * @param http_status expected HTTP response code.
- * @param authorize_reference reference to a /reward-autorize CMD
- *        that offers a reward id to pick up.
- * @param amounts array of string-defined amounts that specifies
- *        which denominations will be accepted for rewardping.
- * @return the command
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_reward_pickup (const char *label,
-                                 const char *merchant_url,
-                                 unsigned int http_status,
-                                 const char *authorize_reference,
-                                 const char **amounts);
-
-
 /**
  * Run a command to fetch the KYC status of a merchant.
  *
@@ -1478,7 +1170,7 @@ TALER_TESTING_cmd_reward_pickup (const char *label,
  *        of the bank account to check KYC for; NULL to check all accounts
  * @param exchange_url base URL of the exchange to check KYC status for
  * @param expected_http_status expected HTTP status
- * @param expected_aml_state expected AML state (only effecitve if @e expected_http_status is #MHD_HTTP_OK)
+ * @param expected_aml_state expected AML state (only effective if @e expected_http_status is #MHD_HTTP_OK)
  * @return the command
  */
 struct TALER_TESTING_Command
@@ -1902,6 +1594,17 @@ TALER_TESTING_cmd_webhook (const char *label,
                            const char *config_filename);
 
 
+/**
+ * Command to run the 'taler-merchant-depositcheck' program.
+ *
+ * @param label command label.
+ * @param config_filename configuration file used by the deposit check helper.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_depositcheck (const char *label,
+                                const char *config_filename);
+
+
 /**
  * Command to run the 'taler-merchant-exchange' program.
  *
@@ -1970,7 +1673,6 @@ TALER_TESTING_cmd_checkserver2 (const char *label,
 // FIXME: rename: refund_entry->refund_detail
 #define TALER_MERCHANT_TESTING_SIMPLE_TRAITS(op) \
   op (claim_nonce, const struct GNUNET_CRYPTO_EddsaPublicKey) \
-  op (reward_id, const struct TALER_RewardIdentifierP) \
   op (pickup_id, const struct TALER_PickupIdentifierP) \
   op (instance_name, const char) \
   op (instance_id, const char) \
