@@ -34,12 +34,12 @@
  * Which version of the Taler protocol is implemented
  * by this library?  Used to determine compatibility.
  */
-#define MERCHANT_PROTOCOL_CURRENT 5
+#define MERCHANT_PROTOCOL_CURRENT 9
 
 /**
- * How many configs are we backwards compatible with?
+ * How many configs are we backwards-compatible with?
  */
-#define MERCHANT_PROTOCOL_AGE 0
+#define MERCHANT_PROTOCOL_AGE 4
 
 
 /**
@@ -105,10 +105,19 @@ handle_config_finished (void *cls,
   case MHD_HTTP_OK:
     {
       const json_t *jcs;
+      const json_t *exchanges = NULL;
+      struct TALER_MERCHANT_ExchangeConfigInfo *eci = NULL;
+      unsigned int num_eci = 0;
       struct TALER_JSON_ProtocolVersion pv;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_object_const ("currencies",
                                        &jcs),
+        /* Optional for v5 compatibility, remove
+           once we reduce age! */
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_array_const ("exchanges",
+                                        &exchanges),
+          NULL),
         GNUNET_JSON_spec_string ("currency",
                                  &cr.details.ok.ci.currency),
         TALER_JSON_spec_version ("version",
@@ -124,6 +133,7 @@ handle_config_finished (void *cls,
                              spec,
                              NULL, NULL))
       {
+        GNUNET_break_op (0);
         cr.hr.http_status = 0;
         cr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
       }
@@ -143,6 +153,39 @@ handle_config_finished (void *cls,
             cr.details.ok.compat |= TALER_MERCHANT_VC_INCOMPATIBLE;
         }
       }
+      if (NULL != exchanges)
+      {
+        num_eci = json_object_size (exchanges);
+        eci = GNUNET_new_array (num_eci,
+                                struct TALER_MERCHANT_ExchangeConfigInfo);
+        for (unsigned int i = 0; i<num_eci; i++)
+        {
+          struct TALER_MERCHANT_ExchangeConfigInfo *ei = &eci[i];
+          const json_t *ej = json_array_get (exchanges,
+                                             i);
+          struct GNUNET_JSON_Specification ispec[] = {
+            GNUNET_JSON_spec_string ("currency",
+                                     &ei->currency),
+            GNUNET_JSON_spec_string ("base_url",
+                                     &ei->base_url),
+            GNUNET_JSON_spec_fixed_auto ("master_pub",
+                                         &ei->master_pub),
+            GNUNET_JSON_spec_end ()
+          };
+
+          if (GNUNET_OK !=
+              GNUNET_JSON_parse (ej,
+                                 ispec,
+                                 NULL, NULL))
+          {
+            GNUNET_break_op (0);
+            cr.hr.http_status = 0;
+            cr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+            GNUNET_free (eci);
+            break;
+          }
+        }
+      }
       {
         unsigned int nspec = json_object_size (jcs);
         struct TALER_CurrencySpecification *cspecs;
@@ -154,11 +197,14 @@ handle_config_finished (void *cls,
                                    struct TALER_CurrencySpecification);
         cr.details.ok.num_cspecs = nspec;
         cr.details.ok.cspecs = cspecs;
+        cr.details.ok.num_exchanges = num_eci;
+        cr.details.ok.exchanges = eci;
         json_object_foreach ((json_t *) jcs, curr, obj)
         {
           struct TALER_CurrencySpecification *cs = &cspecs[off++];
           struct GNUNET_JSON_Specification cspec[] = {
             TALER_JSON_spec_currency_specification (curr,
+                                                    curr,
                                                     cs),
             GNUNET_JSON_spec_end ()
           };
@@ -168,13 +214,16 @@ handle_config_finished (void *cls,
                                  cspec,
                                  NULL, NULL))
           {
+            GNUNET_break_op (0);
             cr.hr.http_status = 0;
             cr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+            GNUNET_free (eci);
             break;
           }
         }
         vgh->cb (vgh->cb_cls,
                  &cr);
+        GNUNET_free (eci);
         TALER_CONFIG_free_currencies (nspec,
                                       cspecs);
       }
