@@ -91,18 +91,17 @@ parse_kyc (struct TALER_MERCHANT_KycGetHandle *kyc,
           sizeof (pending_kycs));
   for (unsigned int i = 0; i<num_pends; i++)
   {
-    uint32_t status;
     struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_mark_optional (
-        GNUNET_JSON_spec_string ("kyc_url",
+        TALER_JSON_spec_web_url ("kyc_url",
                                  &pending_kycs[i].kyc_url),
         NULL),
-      GNUNET_JSON_spec_uint32 ("aml_status",
-                               &status),
-      GNUNET_JSON_spec_string ("exchange_url",
+      TALER_JSON_spec_aml_decision ("aml_status",
+                                    &pending_kycs[i].aml_status),
+      TALER_JSON_spec_web_url ("exchange_url",
                                &pending_kycs[i].exchange_url),
-      GNUNET_JSON_spec_string ("payto_uri",
-                               &pending_kycs[i].payto_uri),
+      TALER_JSON_spec_payto_uri ("payto_uri",
+                                 &pending_kycs[i].payto_uri),
       GNUNET_JSON_spec_end ()
     };
 
@@ -115,18 +114,15 @@ parse_kyc (struct TALER_MERCHANT_KycGetHandle *kyc,
       GNUNET_break (0);
       return GNUNET_SYSERR;
     }
-    pending_kycs[i].aml_status
-      = (enum TALER_AmlDecisionState) status;
   }
   for (unsigned int i = 0; i<num_touts; i++)
   {
     uint32_t hs;
-    uint32_t ec;
     struct GNUNET_JSON_Specification spec[] = {
-      GNUNET_JSON_spec_string ("exchange_url",
+      TALER_JSON_spec_web_url ("exchange_url",
                                &timeout_kycs[i].exchange_url),
-      GNUNET_JSON_spec_uint32 ("exchange_code",
-                               &ec),
+      TALER_JSON_spec_ec ("exchange_code",
+                          &timeout_kycs[i].exchange_code),
       GNUNET_JSON_spec_uint32 ("exchange_http_status",
                                &hs),
       GNUNET_JSON_spec_end ()
@@ -142,7 +138,6 @@ parse_kyc (struct TALER_MERCHANT_KycGetHandle *kyc,
       return GNUNET_SYSERR;
     }
     timeout_kycs[i].exchange_http_status = (unsigned int) hs;
-    timeout_kycs[i].exchange_code = (enum TALER_ErrorCode) ec;
   }
   kr->details.kyc_status.pending_kycs = pending_kycs;
   kr->details.kyc_status.timeout_kycs = timeout_kycs;
@@ -267,17 +262,19 @@ kyc_get (struct GNUNET_CURL_Context *ctx,
   struct TALER_MERCHANT_KycGetHandle *kyc;
   CURL *eh;
   char timeout_ms[32];
+  unsigned int tms;
 
   kyc = GNUNET_new (struct TALER_MERCHANT_KycGetHandle);
   kyc->ctx = ctx;
   kyc->cb = cb;
   kyc->cb_cls = cb_cls;
+  tms = (unsigned int) (timeout.rel_value_us
+                        / GNUNET_TIME_UNIT_MILLISECONDS.
+                        rel_value_us);
   GNUNET_snprintf (timeout_ms,
                    sizeof (timeout_ms),
-                   "%llu",
-                   (unsigned long long) (timeout.rel_value_us
-                                         / GNUNET_TIME_UNIT_MILLISECONDS.
-                                         rel_value_us));
+                   "%u",
+                   tms);
   kyc->url = TALER_url_join (url,
                              "kyc",
                              "h_wire",
@@ -305,6 +302,13 @@ kyc_get (struct GNUNET_CURL_Context *ctx,
               "Requesting URL '%s'\n",
               kyc->url);
   eh = TALER_MERCHANT_curl_easy_get_ (kyc->url);
+  if (0 != tms)
+  {
+    GNUNET_break (CURLE_OK ==
+                  curl_easy_setopt (eh,
+                                    CURLOPT_TIMEOUT_MS,
+                                    (long) (tms + 100L)));
+  }
   kyc->job = GNUNET_CURL_job_add (ctx,
                                   eh,
                                   &handle_get_kyc_finished,
@@ -328,7 +332,7 @@ TALER_MERCHANT_kyc_get (struct GNUNET_CURL_Context *ctx,
                    "%sprivate/",
                    backend_url);
   return kyc_get (ctx,
-                  url,
+                  url, /* consumed! */
                   h_wire,
                   exchange_url,
                   timeout,
@@ -354,7 +358,7 @@ TALER_MERCHANT_management_kyc_get (struct GNUNET_CURL_Context *ctx,
                    backend_url,
                    instance_id);
   return kyc_get (ctx,
-                  url,
+                  url, /* consumed! */
                   h_wire,
                   exchange_url,
                   timeout,
