@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018, 2020 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -67,19 +67,21 @@ struct TALER_MERCHANT_ProductsGetHandle
 /**
  * Parse product information from @a ia.
  *
+ * @param json overall JSON reply
  * @param ia JSON array (or NULL!) with product data
  * @param pgh operation handle
  * @return #GNUNET_OK on success
  */
-static int
-parse_products (const json_t *ia,
+static enum GNUNET_GenericReturnValue
+parse_products (const json_t *json,
+                const json_t *ia,
                 struct TALER_MERCHANT_ProductsGetHandle *pgh)
 {
   unsigned int ies_len = json_array_size (ia);
-  struct TALER_MERCHANT_InventoryEntry ies[ies_len];
+  struct TALER_MERCHANT_InventoryEntry ies[GNUNET_NZL (ies_len)];
   size_t index;
   json_t *value;
-  int ret;
+  enum GNUNET_GenericReturnValue ret;
 
   ret = GNUNET_OK;
   json_array_foreach (ia, index, value) {
@@ -104,14 +106,15 @@ parse_products (const json_t *ia,
   }
   if (GNUNET_OK == ret)
   {
-    struct TALER_MERCHANT_HttpResponse hr = {
-      .http_status = MHD_HTTP_OK
+    struct TALER_MERCHANT_GetProductsResponse gpr = {
+      .hr.http_status = MHD_HTTP_OK,
+      .hr.reply = json,
+      .details.ok.products_length = ies_len,
+      .details.ok.products = ies
     };
 
     pgh->cb (pgh->cb_cls,
-             &hr,
-             ies_len,
-             ies);
+             &gpr);
     pgh->cb = NULL; /* just to be sure */
   }
   return ret;
@@ -133,9 +136,9 @@ handle_get_products_finished (void *cls,
 {
   struct TALER_MERCHANT_ProductsGetHandle *pgh = cls;
   const json_t *json = response;
-  struct TALER_MERCHANT_HttpResponse hr = {
-    .http_status = (unsigned int) response_code,
-    .reply = json
+  struct TALER_MERCHANT_GetProductsResponse gpr = {
+    .hr.http_status = (unsigned int) response_code,
+    .hr.reply = json
   };
 
   pgh->job = NULL;
@@ -146,10 +149,10 @@ handle_get_products_finished (void *cls,
   {
   case MHD_HTTP_OK:
     {
-      json_t *products;
+      const json_t *products;
       struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_json ("products",
-                               &products),
+        GNUNET_JSON_spec_array_const ("products",
+                                      &products),
         GNUNET_JSON_spec_end ()
       };
 
@@ -158,48 +161,39 @@ handle_get_products_finished (void *cls,
                              spec,
                              NULL, NULL))
       {
-        hr.http_status = 0;
-        hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        gpr.hr.http_status = 0;
+        gpr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        break;
       }
-      else
+      if (GNUNET_OK ==
+          parse_products (json,
+                          products,
+                          pgh))
       {
-        if ( (! json_is_array (products)) ||
-             (GNUNET_OK ==
-              parse_products (products,
-                              pgh)) )
-        {
-          GNUNET_JSON_parse_free (spec);
-          TALER_MERCHANT_products_get_cancel (pgh);
-          return;
-        }
-        else
-        {
-          hr.http_status = 0;
-          hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
-        }
+        TALER_MERCHANT_products_get_cancel (pgh);
+        return;
       }
-      GNUNET_JSON_parse_free (spec);
+      gpr.hr.http_status = 0;
+      gpr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
       break;
     }
   case MHD_HTTP_UNAUTHORIZED:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    gpr.hr.ec = TALER_JSON_get_error_code (json);
+    gpr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, merchant says we need to authenticate. */
     break;
   default:
     /* unexpected response code */
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    gpr.hr.ec = TALER_JSON_get_error_code (json);
+    gpr.hr.hint = TALER_JSON_get_error_hint (json);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                (int) hr.ec);
+                (int) gpr.hr.ec);
     break;
   }
   pgh->cb (pgh->cb_cls,
-           &hr,
-           0,
-           NULL);
+           &gpr);
   TALER_MERCHANT_products_get_cancel (pgh);
 }
 
