@@ -476,6 +476,8 @@ TALER_MERCHANT_order_pay (
   TALER_MERCHANT_OrderPayCallback pay_cb,
   void *pay_cb_cls)
 {
+  struct GNUNET_HashCode wallet_data_hash;
+  
   if (GNUNET_YES !=
       TALER_amount_cmp_currency (amount,
                                  max_fee))
@@ -483,7 +485,9 @@ TALER_MERCHANT_order_pay (
     GNUNET_break (0);
     return NULL;
   }
-
+  if (NULL != wallet_data)
+    TALER_json_hash (wallet_data,
+                     &wallet_data_hash);
   {
     struct TALER_MERCHANT_PaidCoin pc[num_coins];
 
@@ -510,7 +514,9 @@ TALER_MERCHANT_order_pay (
                                  &fee,
                                  h_wire,
                                  h_contract_terms,
-                                 NULL /* FIXME: compute using wallet_data */,
+                                 (NULL != wallet_data)
+                                 ? &wallet_data_hash
+                                 : NULL,
                                  coin->h_age_commitment,
                                  NULL /* h_extensions! */,
                                  &h_denom_pub,
