@@ -207,21 +207,16 @@ run (void *cls,
     /* KYC: hence nothing happened at the bank yet: */
     TALER_TESTING_cmd_check_bank_empty ("check_bank_empty-2"),
     /* KYC: we don't even know the legitimization UUID yet */
-    TALER_TESTING_cmd_merchant_kyc_get ("kyc-pending",
+    TALER_TESTING_cmd_merchant_kyc_get ("kyc-pending-early",
                                         merchant_url,
                                         NULL,
                                         NULL,
                                         EXCHANGE_URL,
                                         MHD_HTTP_NO_CONTENT,
                                         TALER_AML_NORMAL),
-    /* now we get the legi UUID */
-    TALER_TESTING_cmd_merchant_get_order ("get-order-kyc",
-                                          merchant_url,
-                                          "create-proposal-1",
-                                          TALER_MERCHANT_OSC_PAID,
-                                          false,
-                                          MHD_HTTP_OK,
-                                          NULL),
+    /* now we get the legi UUID by running taler-merchant-depositcheck */
+    TALER_TESTING_cmd_depositcheck ("deposit-check",
+                                    CONFIG_FILE),
     /* Now we should get a status of pending */
     TALER_TESTING_cmd_merchant_kyc_get ("kyc-pending",
                                         merchant_url,
@@ -397,11 +392,11 @@ run (void *cls,
                                                "default",
                                                MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_post_account (
-        "instance-create-default-account",
-        merchant_url,
-        merchant_payto,
-        NULL, NULL,
-        MHD_HTTP_OK),
+      "instance-create-default-account",
+      merchant_url,
+      merchant_payto,
+      NULL, NULL,
+      MHD_HTTP_OK),
     TALER_TESTING_cmd_batch ("pay",
                              pay),
     TALER_TESTING_cmd_batch ("aml",
