@@ -33,7 +33,6 @@ function my_cleanup()
 
 . setup.sh
 
-# Launch only the merchant.
 setup -c test_template.conf -m
 CONF="test_template.conf.edited"
 LAST_RESPONSE=$(mktemp -p "${TMPDIR:-/tmp}" test_response.conf-XXXXXX)
@@ -52,7 +51,7 @@ fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     -H 'Authorization: Bearer secret-token:new_value' \
-    http://localhost:9966/instances/default/private/accounts \
+    http://localhost:9966/private/accounts \
     -d '{"payto_uri":"payto://x-taler-bank/localhost:8082/43"}' \
     -w "%{http_code}" -s -o /dev/null)
 
@@ -69,6 +68,8 @@ kill -TERM "$SETUP_PID"
 wait
 unset SETUP_PID
 
+setup -c test_template.conf -ef -u "exchange-account-2"
+
 NEW_SECRET=secret-token:different_value
 
 taler-merchant-httpd -a "${NEW_SECRET}" -c "${CONF}" -L DEBUG 2> taler-merchant-httpd.log &
@@ -95,7 +96,7 @@ fi
 
 echo -n "Creating order to test auth is ok..." >&2
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
-    'http://localhost:9966/instances/default/private/orders' \
+    'http://localhost:9966/private/orders' \
     -H 'Authorization: Bearer '"$NEW_SECRET" \
     -d '{"order":{"amount":"TESTKUDOS:1","summary":"payme"}}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
@@ -109,7 +110,7 @@ fi
 ORDER_ID=$(jq -e -r .order_id < "$LAST_RESPONSE")
 TOKEN=$(jq -e -r .token < "$LAST_RESPONSE")
 
-STATUS=$(curl "http://localhost:9966/instances/default/private/orders/${ORDER_ID}" \
+STATUS=$(curl "http://localhost:9966/private/orders/${ORDER_ID}" \
      -H 'Authorization: Bearer '"$NEW_SECRET" \
      -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
