@@ -20,9 +20,18 @@ else
     ACCOUNT="exchange-account-1"
     BANK_FLAGS="-ns -d iban -u $ACCOUNT"
     BANK_URL="http://localhost:18082/"
+    echo -n "Testing for libeufin-bank"
+    libeufin-bank --help >/dev/null </dev/null || exit_skip " MISSING"
+    echo " FOUND"
+
 fi
 
 . setup.sh
+
+echo -n "Testing for taler-harness"
+taler-harness --help >/dev/null </dev/null || exit_skip " MISSING"
+echo " FOUND"
+
 # Launch exchange, merchant and bank.
 setup -c "test_template.conf" \
       -em \
@@ -106,7 +115,7 @@ fi
 # create with 2 bank account addresses
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     -H 'Authorization: Bearer secret-token:super_secret' \
-    http://localhost:9966/instances/default/private/accounts \
+    http://localhost:9966/private/accounts \
     -d '{"payto_uri":"'"$FORTYTHREE"'"}' \
     -w "%{http_code}" -s -o /dev/null)
 
@@ -116,7 +125,7 @@ then
 fi
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     -H 'Authorization: Bearer secret-token:super_secret' \
-    http://localhost:9966/instances/default/private/accounts \
+    http://localhost:9966/private/accounts \
     -d '{"payto_uri":"payto://iban/SANDBOXX/DE270744?receiver-name=Forty+Four"}' \
     -w "%{http_code}" -s -o /dev/null)
 
@@ -129,7 +138,7 @@ fi
 echo "Ok"
 
 echo -n "Get accounts..."
-STATUS=$(curl http://localhost:9966/instances/default/private/accounts \
+STATUS=$(curl http://localhost:9966/private/accounts \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 PAY_URI=$(jq -r .accounts[1].payto_uri < "$LAST_RESPONSE")
 H_WIRE=$(jq -r .accounts[1].h_wire < "$LAST_RESPONSE")
@@ -144,7 +153,7 @@ echo "OK"
 echo -n "Deleting one account ..."
 STATUS=$(curl -H "Content-Type: application/json" -X PATCH \
     -H 'Authorization: Bearer secret-token:super_secret' \
-    "http://localhost:9966/instances/default/private/accounts/${H_WIRE}" \
+    "http://localhost:9966/private/accounts/${H_WIRE}" \
     -X DELETE \
     -w "%{http_code}" -s -o /dev/null)
 
@@ -161,7 +170,7 @@ RANDOM_IMG='data:image/png;base64,abcdefg'
 #
 
 echo -n "Creating order without TOKEN..."
-STATUS=$(curl 'http://localhost:9966/instances/default/private/orders' \
+STATUS=$(curl 'http://localhost:9966/private/orders' \
     -d '{"create_token":false,"order":{"amount":"TESTKUDOS:7","summary":"3","products":[{"description":"desct","image":"'"$RANDOM_IMG"'","price":"TESTKUDOS:1","taxes":[],"unit":"u","quantity":1}]}}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
@@ -197,7 +206,7 @@ echo "OK"
 #
 
 echo -n "Creating order without TOKEN and fullfilment URL..."
-STATUS=$(curl 'http://localhost:9966/instances/default/private/orders' \
+STATUS=$(curl 'http://localhost:9966/private/orders' \
     -d '{"create_token":false,"order":{"fulfillment_url":"go_here_please", "amount":"TESTKUDOS:7","summary":"3","products":[{"description":"desct","image":"'"$RANDOM_IMG"'","price":"TESTKUDOS:1","taxes":[],"unit":"u","quantity":1}]}}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
@@ -239,7 +248,7 @@ echo "OK"
 #
 
 echo -n "Creating order with non-inventory products..."
-STATUS=$(curl 'http://localhost:9966/instances/default/private/orders' \
+STATUS=$(curl 'http://localhost:9966/private/orders' \
     -d '{"order":{"amount":"TESTKUDOS:7","summary":"3","products":[{"description":"desct","image":"'"$RANDOM_IMG"'","price":"TESTKUDOS:1","taxes":[],"unit":"u","quantity":1}]}}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
@@ -281,7 +290,7 @@ echo "OK"
 #
 
 echo -n "Creating product..."
-STATUS=$(curl 'http://localhost:9966/instances/default/private/products' \
+STATUS=$(curl 'http://localhost:9966/private/products' \
     -d '{"product_id":"2","description":"product with id 2 and price :15","price":"TESTKUDOS:15","total_stock":2,"description_i18n":{},"unit":"","image":"'$RANDOM_IMG'","taxes":[],"address":{},"next_restock":{"t_s":"never"}}' \
     -w "%{http_code}" -s -o /dev/null)
 
@@ -292,7 +301,7 @@ fi
 echo "OK"
 
 echo -n "Creating order with inventory products..."
-STATUS=$(curl 'http://localhost:9966/instances/default/private/orders' \
+STATUS=$(curl 'http://localhost:9966/private/orders' \
     -d '{"order":{"amount":"TESTKUDOS:7","summary":"3"},"inventory_products":[{"product_id":"2","quantity":1}]}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
@@ -330,7 +339,7 @@ echo "OK"
 #
 
 
-STATUS=$(curl 'http://localhost:9966/instances/default/private/products' \
+STATUS=$(curl 'http://localhost:9966/private/products' \
     -d '{"product_id":"1","description":"product with id 1 and price :15","price":"USD:15","total_stock":1,"description_i18n":{},"unit":"","image":"","taxes":[],"address":{},"next_restock":{"t_s":"never"}}' \
     -w "%{http_code}" -s -o /dev/null)
 
@@ -344,7 +353,7 @@ fi
 #
 
 echo -n "Creating order to be paid..."
-STATUS=$(curl 'http://localhost:9966/instances/default/private/orders' \
+STATUS=$(curl 'http://localhost:9966/private/orders' \
     -d '{"order":{"amount":"TESTKUDOS:1","summary":"payme"},"inventory_products":[{"product_id":"2","quantity":1}]}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
@@ -357,7 +366,7 @@ fi
 ORDER_ID=$(jq -e -r .order_id < "$LAST_RESPONSE")
 TOKEN=$(jq -e -r .token < "$LAST_RESPONSE")
 
-STATUS=$(curl "http://localhost:9966/instances/default/private/orders/${ORDER_ID}" \
+STATUS=$(curl "http://localhost:9966/private/orders/${ORDER_ID}" \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -378,7 +387,7 @@ taler-wallet-cli --no-throttle --wallet-db="$WALLET_DB" run-until-done 2> wallet
 NOW2=$(date +%s)
 echo " OK (took $(( NOW2 - NOW )) secs )"
 
-STATUS=$(curl "http://localhost:9966/instances/default/private/orders/${ORDER_ID}" \
+STATUS=$(curl "http://localhost:9966/private/orders/${ORDER_ID}" \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -421,27 +430,11 @@ echo " DONE"
 
 echo -n "Obtaining wire transfer details from bank ($USE_FAKEBANK)..."
 
-if [ 1 = "$USE_FAKEBANK" ]
-then
-    BANKDATA="$(curl 'http://localhost:8082/accounts/exchange/taler-wire-gateway/history/outgoing?delta=1' -s)"
-    WTID=$(echo "$BANKDATA" | jq -r .outgoing_transactions[0].wtid)
-    WURL=$(echo "$BANKDATA" | jq -r .outgoing_transactions[0].exchange_base_url)
-    CREDIT_AMOUNT=$(echo "$BANKDATA" | jq -r .outgoing_transactions[0].amount)
-    TARGET_PAYTO=$(echo "$BANKDATA" | jq -r .outgoing_transactions[0].credit_account)
-else
-    # Emulating the previous pybank-based logic of getting
-    # the wire transfer information _via the exchange_ bank
-    # account.  NOTE: grabbing tx == 0, since the latest
-    # transaction appear first in the bank's history.
-    BANKDATA=$(get_bankaccount_transactions exchange x | jq '.transactions[0]')
-    SUBJECT=$(echo "$BANKDATA" | jq -r .subject)
-    WTID=$(echo "$SUBJECT" | awk '{print $1}')
-    WURL=$(echo "$SUBJECT" | awk '{print $2}')
-    CREDIT_AMOUNT="$(echo "$BANKDATA" | jq -r .currency):$(echo "$BANKDATA" | jq -r .amount)"
-    TARGET=$(echo "$BANKDATA" | jq -r .creditorIban)
-    # 'TARGET' is now the IBAN.
-    TARGET_PAYTO="payto://iban/SANDBOXX/$TARGET?receiver-name=Forty+Three"
-fi
+BANKDATA="$(curl 'http://localhost:8082/accounts/exchange/taler-wire-gateway/history/outgoing?delta=1' -s)"
+WTID=$(echo "$BANKDATA" | jq -r .outgoing_transactions[0].wtid)
+WURL=$(echo "$BANKDATA" | jq -r .outgoing_transactions[0].exchange_base_url)
+CREDIT_AMOUNT=$(echo "$BANKDATA" | jq -r .outgoing_transactions[0].amount)
+TARGET_PAYTO=$(echo "$BANKDATA" | jq -r .outgoing_transactions[0].credit_account)
 
 if [ "$EXCHANGE_URL" != "$WURL" ]
 then
@@ -454,7 +447,7 @@ set +e
 
 echo -n "Notifying merchant of bogus wire transfer ..."
 
-STATUS=$(curl 'http://localhost:9966/instances/default/private/transfers' \
+STATUS=$(curl 'http://localhost:9966/private/transfers' \
     -d '{"credit_amount":"'"$CREDIT_AMOUNT"'1","wtid":"'"$WTID"'","payto_uri":"'"$TARGET_PAYTO"'","exchange_url":"'"$WURL"'"}' \
     -m 3 \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
@@ -469,7 +462,7 @@ echo "OK"
 
 echo -n "Notifying merchant of correct wire transfer (conflicting with old data)..."
 
-STATUS=$(curl 'http://localhost:9966/instances/default/private/transfers' \
+STATUS=$(curl 'http://localhost:9966/private/transfers' \
     -d '{"credit_amount":"'"$CREDIT_AMOUNT"'","wtid":"'"$WTID"'","payto_uri":"'"$TARGET_PAYTO"'","exchange_url":"'"$WURL"'"}' \
     -m 3 \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
@@ -484,9 +477,9 @@ echo " OK"
 
 echo -n "Deleting bogus wire transfer ..."
 
-TID=$(curl -s http://localhost:9966/instances/default/private/transfers | jq -r .transfers[0].transfer_serial_id)
+TID=$(curl -s http://localhost:9966/private/transfers | jq -r .transfers[0].transfer_serial_id)
 STATUS=$(curl -H "Content-Type: application/json" -X DELETE \
-    "http://localhost:9966/instances/default/private/transfers/$TID" \
+    "http://localhost:9966/private/transfers/$TID" \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "204" ]
@@ -496,7 +489,7 @@ then
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X DELETE \
-    "http://localhost:9966/instances/default/private/transfers/$TID" \
+    "http://localhost:9966/private/transfers/$TID" \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 if [ "$STATUS" != "404" ]
 then
@@ -508,7 +501,7 @@ echo " OK"
 
 echo -n "Notifying merchant of correct wire transfer (now working)..."
 
-STATUS=$(curl 'http://localhost:9966/instances/default/private/transfers' \
+STATUS=$(curl 'http://localhost:9966/private/transfers' \
     -d '{"credit_amount":"'"$CREDIT_AMOUNT"'","wtid":"'"$WTID"'","payto_uri":"'"$TARGET_PAYTO"'","exchange_url":"'"$WURL"'"}' \
     -m 3 \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
@@ -527,7 +520,7 @@ set -e
 
 # Test idempotence: do it again!
 
-STATUS=$(curl 'http://localhost:9966/instances/default/private/transfers' \
+STATUS=$(curl 'http://localhost:9966/private/transfers' \
     -d '{"credit_amount":"'"$CREDIT_AMOUNT"'","wtid":"'"$WTID"'","payto_uri":"'"$TARGET_PAYTO"'","exchange_url":"'"$WURL"'"}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
@@ -546,7 +539,7 @@ echo " OK"
 
 echo -n "Fetching wire transfers ..."
 
-STATUS=$(curl 'http://localhost:9966/instances/default/private/transfers' \
+STATUS=$(curl 'http://localhost:9966/private/transfers' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -566,7 +559,7 @@ fi
 echo "OK"
 
 echo -n "Checking order status ..."
-STATUS=$(curl "http://localhost:9966/instances/default/private/orders/${ORDER_ID}?transfer=YES" \
+STATUS=$(curl "http://localhost:9966/private/orders/${ORDER_ID}?transfer=YES" \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 if [ "$STATUS" != "200" ]
 then
@@ -602,7 +595,6 @@ then
 else
     ACCOUNT_PASSWORD="fortythree:x"
     BANK_HOST="localhost:18082"
-    # Can be replaced by the libeufin-cli way.
     STATUS=$(curl "http://$ACCOUNT_PASSWORD@$BANK_HOST/accounts/fortythree" \
                   -w "%{http_code}" -s -o "$LAST_RESPONSE")
     if [ "$STATUS" != "200" ]
@@ -621,7 +613,7 @@ echo " OK"
 
 echo -n "Getting information about kyc ..."
 STATUS=$(curl -H "Content-Type: application/json" -X GET \
-    http://localhost:9966/instances/default/private/kyc \
+    http://localhost:9966/private/kyc \
     -w "%{http_code}" -s -o /dev/null)
 if [ "$STATUS" != "204" ]
 then
