@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2020 Taler Systems SA
+  Copyright (C) 2020-2024 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -536,7 +536,6 @@ merchant_get_order_run (void *cls,
     gos->merchant_url,
     order_id,
     NULL,
-    true,
     GNUNET_TIME_UNIT_ZERO,
     &merchant_get_order_cb,
     gos);
@@ -858,7 +857,6 @@ merchant_poll_order_start_run (void *cls,
     pos->merchant_url,
     pos->order_id,
     NULL,
-    false,
     pos->timeout,
     &merchant_poll_order_cb,
     pos);
